@@ -3,7 +3,7 @@
 #
 # support for colour maps and other lookup tables
 #
-# $Revision: 1.31 $ $Date: 2014/04/17 03:58:58 $
+# $Revision: 1.37 $ $Date: 2016/02/16 01:39:12 $
 #
 
 colourmap <- function(col, ..., range=NULL, breaks=NULL, inputs=NULL) {
@@ -12,7 +12,7 @@ colourmap <- function(col, ..., range=NULL, breaks=NULL, inputs=NULL) {
     f <- lut()
   } else {
     ## validate colour data 
-    h <- col2hex(col)
+    col2hex(col)
     ## store without conversion
     f <- lut(col, ..., range=range, breaks=breaks, inputs=inputs)
   }
@@ -269,7 +269,7 @@ plot.colourmap <- local({
 
     # .......... initialise plot ...............................
     if(!add)
-      do.call.matched("plot.default",
+      do.call.matched(plot.default,
                       resolve.defaults(list(x=xlim, y=ylim,
                                             type="n", main=main,
                                             axes=FALSE, xlab="", ylab="",
@@ -286,7 +286,7 @@ plot.colourmap <- local({
         z <- matrix(1, 1, 1)
         for(i in 1:n) {
           x <- c(xleft[i], xright[i])
-          do.call.matched("image.default",
+          do.call.matched(image.default,
                       resolve.defaults(list(x=x, y=y, z=z, add=TRUE),
                                        list(...),
                                        list(col=col[i])),
@@ -301,7 +301,7 @@ plot.colourmap <- local({
         z <- matrix(1, 1, 1)
         for(i in 1:n) {
           y <- c(ylow[i], yupp[i])
-          do.call.matched("image.default",
+          do.call.matched(image.default,
                       resolve.defaults(list(x=x, y=y, z=z, add=TRUE),
                                        list(...),
                                        list(col=col[i])),
@@ -322,7 +322,7 @@ plot.colourmap <- local({
         z <- matrix(v, nrow=1)
         x <- xlim
       }
-      do.call.matched("image.default",
+      do.call.matched(image.default,
                       resolve.defaults(list(x=x, y=y, z=z, add=TRUE),
                                        list(...),
                                        list(breaks=bks, col=col)),
@@ -349,7 +349,7 @@ plot.colourmap <- local({
         # don't draw axis lines if plotting separate blocks
         lwd0 <- if(separate) 0 else 1
         # draw axis
-        do.call.matched("axis",
+        do.call.matched(graphics::axis,
                         resolve.defaults(list(...),
                                          list(side = 1, pos = pos, at = at),
                                          list(labels=la, lwd=lwd0)),
@@ -375,7 +375,7 @@ plot.colourmap <- local({
         # draw labels horizontally if plotting separate blocks
         las0 <- if(separate) 1 else 0
         # draw axis
-        do.call.matched("axis",
+        do.call.matched(graphics::axis,
                         resolve.defaults(list(...),
                                          list(side=4, pos=pos, at=at),
                                          list(labels=la, lwd=lwd0, las=las0)),
@@ -409,7 +409,7 @@ interp.colourmap <- function(m, n=512) {
     xknots <- (bks[-1] + bks[-nb])/2
   }
   # corresponding colours in hsv coordinates
-  yknots.hsv <- rgb2hsv(col2rgb(st$outputs))
+  yknots.hsv <- rgb2hsva(col2rgb(st$outputs, alpha=TRUE))
   # transform 'hue' from polar to cartesian coordinate
   # divide domain into n equal intervals
   xrange <- range(xknots)
@@ -423,67 +423,108 @@ interp.colourmap <- function(m, n=512) {
   yy.huex <- approx(x=xknots, y=cos(yknots.hue), xout=xx)$y
   yy.huey <- approx(x=xknots, y=sin(yknots.hue), xout=xx)$y
   yy.hue <- (atan2(yy.huey, yy.huex)/(2 * pi)) %% 1
-  # form colours using hue, sat, val
-  yy <- hsv(yy.hue, yy.sat, yy.val)
+  # handle transparency
+  yknots.alpha <- yknots.hsv["alpha", ]
+  if(all(yknots.alpha == 1)) {
+    ## opaque colours: form using hue, sat, val
+    yy <- hsv(yy.hue, yy.sat, yy.val)
+  } else {
+    ## transparent colours: interpolate alpha
+    yy.alpha <- approx(x=xknots, y=yknots.alpha, xout=xx)$y
+    ## form colours using hue, sat, val, alpha
+    yy <- hsv(yy.hue, yy.sat, yy.val, yy.alpha)    
+  }
   # done
   f <- colourmap(yy, breaks=xbreaks)
   return(f)
 }
 
-tweak.colourmap <- function(m, col, ..., inputs=NULL, range=NULL) {
-  if(!inherits(m, "colourmap"))
-    stop("m should be a colourmap")
-  if(is.null(inputs) && is.null(range))
-    stop("Specify either inputs or range")
-  if(!is.null(inputs) && !is.null(range))
-    stop("Do not specify both inputs and range")
-  # determine indices of colours to be changed
-  if(!is.null(inputs)) {
-    ix <- m(inputs, what="index")
-  } else {
-    if(!(is.numeric(range) && length(range) == 2 && diff(range) > 0))
-      stop("range should be a numeric vector of length 2 giving (min, max)")
-    if(length(col2hex(col)) != 1)
-      stop("When range is given, col should be a single colour value")
-    ixr <- m(range, what="index")
-    ix <- (ixr[1]):(ixr[2])
+interp.colours <- function(x, length.out=512) {
+  y <- colourmap(x, range=c(0,1))
+  z <- interp.colourmap(y, length.out)
+  oo <- attr(z, "stuff")$outputs
+  return(oo)
+}
+
+tweak.colourmap <- local({
+
+  is.hex <- function(z) {
+    is.character(z) &&
+    all(nchar(z, keepNA=TRUE) %in% c(7,9)) &&
+    identical(substr(z, 1, 7), substr(col2hex(z), 1, 7))
   }
-  # reassign colours
-  st <- attr(m, "stuff")
-  outputs <- st$outputs
-  is.hex <- function(z) identical(substr(z, 1, 7), substr(col2hex(z), 1, 7))
-  result.hex <- FALSE
-  if(is.hex(outputs)) {
-    # convert replacement data to hex
-    col <- col2hex(col)
-    result.hex <- TRUE
-  } else if(is.hex(col)) {
-    # convert existing data to hex
-    outputs <- col2hex(outputs)
-    result.hex <- TRUE
-  } else if(!(is.character(outputs) && is.character(col))) {
-    # unrecognised format - convert both to hex
-    outputs <- col2hex(outputs)
-    col <- col2hex(col)
-    result.hex <- TRUE
-  }
-  if(result.hex) {
-    # hex codes may be 7 or 9 characters
-    outlen <- nchar(outputs)
-    collen <- nchar(col)
-    if(length(unique(c(outlen, collen))) > 1) {
-      # convert all to 9 characters
-      if(any(bad <- (outlen == 7))) 
-        outputs[bad] <- paste0(outputs[bad], "FF")
-      if(any(bad <- (collen == 7))) 
-        col[bad] <- paste0(col[bad], "FF")
+
+  tweak.colourmap <- function(m, col, ..., inputs=NULL, range=NULL) {
+    if(!inherits(m, "colourmap"))
+      stop("m should be a colourmap")
+    if(is.null(inputs) && is.null(range)) 
+      stop("Specify either inputs or range")
+    if(!is.null(inputs) && !is.null(range))
+      stop("Do not specify both inputs and range")
+    ## determine indices of colours to be changed
+    if(!is.null(inputs)) {
+      ix <- m(inputs, what="index")
+    } else {
+      if(!(is.numeric(range) && length(range) == 2 && diff(range) > 0))
+        stop("range should be a numeric vector of length 2 giving (min, max)")
+      if(length(col2hex(col)) != 1)
+        stop("When range is given, col should be a single colour value")
+      ixr <- m(range, what="index")
+      ix <- (ixr[1]):(ixr[2])
+    }
+    ## reassign colours
+    st <- attr(m, "stuff")
+    outputs <- st$outputs
+    result.hex <- FALSE
+    if(is.hex(outputs)) {
+      ## convert replacement data to hex
+      col <- col2hex(col)
+      result.hex <- TRUE
+    } else if(is.hex(col)) {
+      ## convert existing data to hex
+      outputs <- col2hex(outputs)
+      result.hex <- TRUE
+    } else if(!(is.character(outputs) && is.character(col))) {
+      ## unrecognised format - convert both to hex
+      outputs <- col2hex(outputs)
+      col <- col2hex(col)
+      result.hex <- TRUE
+    }
+    if(result.hex) {
+      ## hex codes may be 7 or 9 characters
+      outlen <- nchar(outputs)
+      collen <- nchar(col)
+      if(length(unique(c(outlen, collen))) > 1) {
+        ## convert all to 9 characters
+        if(any(bad <- (outlen == 7))) 
+          outputs[bad] <- paste0(outputs[bad], "FF")
+        if(any(bad <- (collen == 7))) 
+          col[bad] <- paste0(col[bad], "FF")
+      }
     }
+    ## Finally, replace
+    outputs[ix] <- col
+    st$outputs <- outputs
+    attr(m, "stuff") <- st
+    assign("stuff", st, envir=environment(m))
+    return(m)
   }
-  # Finally, replace
-  outputs[ix] <- col
-  st$outputs <- outputs
-  attr(m, "stuff") <- st
-  assign("stuff", st, envir=environment(m))
-  return(m)
+
+  tweak.colourmap
+})
+
+colouroutputs <- function(x) {
+  stopifnot(inherits(x, "colourmap"))
+  attr(x, "stuff")$outputs
+}
+
+"colouroutputs<-" <- function(x, value) {
+  stopifnot(inherits(x, "colourmap"))
+  st <- attr(x, "stuff")
+  col2hex(value) # validates colours
+  st$outputs[] <- value
+  attr(x, "stuff") <- st
+  assign("stuff", st, envir=environment(x))
+  return(x)
 }
 
