@@ -3,9 +3,7 @@
 #
 #  Method for 'density' for point patterns
 #
-#  + bandwidth selection rules bw.diggle, bw.scott
-#
-#  $Revision: 1.66 $    $Date: 2014/04/02 10:31:39 $
+#  $Revision: 1.85 $    $Date: 2016/11/13 01:54:32 $
 #
 
 ksmooth.ppp <- function(x, sigma, ..., edge=TRUE) {
@@ -18,25 +16,52 @@ density.ppp <- local({
 density.ppp <- function(x, sigma=NULL, ...,
                         weights=NULL, edge=TRUE, varcov=NULL,
                         at="pixels", leaveoneout=TRUE,
-                        adjust=1, diggle=FALSE) {
+                        adjust=1, diggle=FALSE, se=FALSE, 
+                        kernel="gaussian",
+                        scalekernel=is.character(kernel),
+                        positive=FALSE) {
   verifyclass(x, "ppp")
 
   output <- pickoption("output location type", at,
                        c(pixels="pixels",
                          points="points"))
+
+  if(!identical(kernel, "gaussian")) {
+    validate2Dkernel(kernel)
+    ## kernel is only partly implemented!
+    if(se)
+      stop("Standard errors are not implemented for non-Gaussian kernel")
+    if(is.function(sigma) || (is.null(sigma) && is.null(varcov)))
+      warning("Bandwidth selection will be based on Gaussian kernel")
+  }
   
   ker <- resolve.2D.kernel(..., sigma=sigma, varcov=varcov, x=x, adjust=adjust)
   sigma <- ker$sigma
   varcov <- ker$varcov
 
-  if(is.expression(weights)) 
+  if(is.im(weights)) {
+    weights <- safelookup(weights, x) # includes warning if NA
+  } else if(is.expression(weights)) 
     weights <- eval(weights, envir=as.data.frame(x), enclos=parent.frame())
   if(length(weights) == 0 || (!is.null(dim(weights)) && nrow(weights) == 0))
     weights <- NULL 
 
+  if(se) {
+    # compute standard error
+    SE <- denspppSEcalc(x, sigma=sigma, varcov=varcov,
+                        ...,
+                        weights=weights, edge=edge, at=output,
+                        leaveoneout=leaveoneout, adjust=adjust,
+                        diggle=diggle)
+    if(positive) SE <- posify(SE)
+  }
+                         
   if(output == "points") {
     # VALUES AT DATA POINTS ONLY
-    result <- densitypointsEngine(x, sigma, varcov=varcov,
+    result <- densitypointsEngine(x, sigma,
+                                  varcov=varcov,
+                                  kernel=kernel,
+                                  scalekernel=scalekernel,
                                   weights=weights, edge=edge,
                                   leaveoneout=leaveoneout,
                                   diggle=diggle, ...)
@@ -45,6 +70,11 @@ density.ppp <- function(x, sigma=NULL, ...,
              underflow=warning("underflow due to very small bandwidth"),
              warning(uhoh))
     }
+    ## constrain values to be positive
+    if(positive) 
+      result <- posify(result)
+    if(se) 
+      result <- list(estimate=result, SE=SE)
     return(result)
   }
   
@@ -53,20 +83,26 @@ density.ppp <- function(x, sigma=NULL, ...,
     # no edge correction
     edg <- NULL
     raw <- second.moment.calc(x, sigma, what="smooth", ...,
+                              kernel=kernel,
+                              scalekernel=scalekernel,
                               weights=weights, varcov=varcov)
     raw <- divide.by.pixelarea(raw) 
     smo <- raw
   } else if(!diggle) {
     # edge correction e(u)
     both <- second.moment.calc(x, sigma, what="smoothedge", ...,
-                              weights=weights, varcov=varcov)
+                               kernel=kernel,
+                               scalekernel=scalekernel,
+                               weights=weights, varcov=varcov)
     raw <- divide.by.pixelarea(both$smooth)
     edg <- both$edge
     smo <- if(is.im(raw)) eval.im(raw/edg) else
-           lapply(raw, function(a,b) eval.im(a/b), b=edg)
+           lapply(raw, divideimage, denom=edg)
   } else {
     # edge correction e(x_i)
-    edg <- second.moment.calc(x, sigma, what="edge", ..., varcov=varcov)
+    edg <- second.moment.calc(x, sigma, what="edge", ...,
+                              scalekernel=scalekernel,
+                              kernel=kernel, varcov=varcov)
     wi <- 1/safelookup(edg, x, warn=FALSE)
     wi[!is.finite(wi)] <- 0
     # edge correction becomes weight attached to points
@@ -80,25 +116,48 @@ density.ppp <- function(x, sigma=NULL, ...,
       newweights <- weights * wi
     }
     raw <- second.moment.calc(x, sigma, what="smooth", ...,
+                              kernel=kernel,
+                              scalekernel=scalekernel,
                               weights=newweights, varcov=varcov)
     raw <- divide.by.pixelarea(raw)
     smo <- raw
   }
 
   result <- if(is.im(smo)) smo[x$window, drop=FALSE]
-            else as.listof(lapply(smo, "[", i=x$window, drop=FALSE))
+            else solapply(smo, "[", i=x$window, drop=FALSE)
 
   # internal use only
-  spill <- list(...)$spill
-  if(!is.null(spill)) 
-    return(list(sigma=sigma, varcov=varcov, raw = raw, edg=edg))
+  spill <- resolve.1.default(list(spill=FALSE), list(...))
+  if(spill)
+    return(list(result=result, sigma=sigma, varcov=varcov, raw = raw, edg=edg))
+
+  # constrain values to be positive
+  if(positive) 
+    result <- posify(result)
 
   # normal return
   attr(result, "sigma") <- sigma
   attr(result, "varcov") <- varcov
+  attr(result, "kernel") <- kernel
+  if(se)
+    result <- list(estimate=result, SE=SE)
   return(result)
 }
 
+divideimage <- function(numer, denom) eval.im(numer/denom)
+
+posify <- function(x, eps=.Machine$double.xmin) {
+  force(eps) # scalpel
+  if(is.im(x)) return(eval.im(pmax(eps, x)))
+  if(inherits(x, "solist")) return(solapply(x, posify, eps=eps))
+  if(is.numeric(x)) return(pmax(eps, x))
+  # data frame or list
+  if(is.list(x) && all(sapply(x, is.numeric)))
+    return(lapply(x, posify, eps=eps))
+  warning("Internal error: posify did not recognise data format")
+  return(x)
+}
+
 divide.by.pixelarea <- function(x) {
   if(is.im(x)) {
     x$v <- x$v/(x$xstep * x$ystep)
@@ -109,42 +168,96 @@ divide.by.pixelarea <- function(x) {
   return(x)
 }
 
+denspppSEcalc <- function(x, sigma, varcov, ...,
+                          weights, edge, diggle, at) {
+  ## Calculate standard error, rather than estimate
+  tau <- taumat <- NULL
+  if(is.null(varcov)) {
+    varconst <- 1/(4 * pi * prod(sigma))
+    tau <- sigma/sqrt(2)
+  } else {
+    varconst <- 1/(4 * pi * sqrt(det(varcov)))
+    taumat <- varcov/2
+  }
+  ## Calculate edge correction weights
+  if(edge) {
+    edgeim <- second.moment.calc(x, sigma, what="edge", ...,
+                                 varcov=varcov)
+    if(diggle || at == "points") {
+      edgeX <- safelookup(edgeim, x, warn=FALSE)
+      diggleX <- 1/edgeX
+      diggleX[!is.finite(diggleX)] <- 0
+    }
+    edgeim <- edgeim[Window(x), drop=FALSE]
+  }
+  ## Perform smoothing
+  if(!edge) {
+    ## no edge correction
+    V <- density(x, sigma=tau, varcov=taumat, ...,
+                 weights=weights, edge=edge, diggle=diggle, at=at)
+  } else if(!diggle) {
+    ## edge correction e(u)
+    V <- density(x, sigma=tau, varcov=taumat, ...,
+                 weights=weights, edge=edge, diggle=diggle, at=at)
+    V <- if(at == "pixels") (V/edgeim) else (V * diggleX)
+  } else {
+    ## Diggle edge correction e(x_i)
+    wts <- diggleX * (weights %orifnull% 1)
+    V <- density(x, sigma=tau, varcov=taumat, ...,
+                 weights=wts, edge=edge, diggle=diggle, at=at)
+  }
+  V <- V * varconst
+  return(sqrt(V))
+}       
+
+
 density.ppp
 
 })
 
 densitypointsEngine <- function(x, sigma, ...,
+                                kernel="gaussian", 
+                                scalekernel=is.character(kernel),
                                 weights=NULL, edge=TRUE, varcov=NULL,
                                 leaveoneout=TRUE, diggle=FALSE,
-                                sorted=FALSE) {
+                                sorted=FALSE, spill=FALSE, cutoff=NULL) {
+  debugging <- spatstat.options("developer")
+  stopifnot(is.logical(leaveoneout))
+
+  validate2Dkernel(kernel)
+  if(is.character(kernel)) kernel <- match2DkernelName(kernel)
+  isgauss <- identical(kernel, "gaussian")
+
+  # constant factor in density computations
   if(is.null(varcov)) {
-    const <- 1/(2 * pi * sigma^2)
+    const <- 1/sigma^2 
   } else {
     detSigma <- det(varcov)
     Sinv <- solve(varcov)
-    const <- 1/(2 * pi * sqrt(detSigma))
+    const <- 1/sqrt(detSigma)
   }
+  if(isgauss) {
+    # absorb leading constant in Gaussian density
+    const <- const/(2 * pi)
+  }
+  
   if(length(weights) == 0 || (!is.null(dim(weights)) && nrow(weights) == 0))
     weights <- NULL
   # Leave-one-out computation
   # cutoff: contributions from pairs of distinct points
   # closer than 8 standard deviations
   sd <- if(is.null(varcov)) sigma else sqrt(sum(diag(varcov)))
-  cutoff <- 8 * sd
-#  nnd <- nndist(x)
-#  nnrange <- range(nnd)
-#  if(nnrange[1] > cutoff) {
-#    npts <- npoints(x)
-#    result <- if(leaveoneout) numeric(npts) else rep.int(const, npts)
-#    attr(result, "sigma") <- sigma
-#    attr(result, "varcov") <- varcov
-#    attr(result, "warnings") <- "underflow"
-#    return(result)
-#  }
-  if(leaveoneout) {
+  if(is.null(cutoff)) 
+    cutoff <- 8 * sd
+  if(debugging)
+    cat(paste("cutoff=", cutoff, "\n"))
+
+  if(leaveoneout && npoints(x) > 1) {
     # ensure each point has its closest neighbours within the cutoff
     nndmax <- maxnndist(x)
     cutoff <- max(2 * nndmax, cutoff)
+    if(debugging)
+      cat(paste("adjusted cutoff=", cutoff, "\n"))
   }
   # validate weights
   if(is.null(weights)) {
@@ -161,7 +274,7 @@ densitypointsEngine <- function(x, sigma, ...,
   # evaluate edge correction weights at points 
   if(edge) {
     win <- x$window
-    if(is.null(varcov) && win$type == "rectangle") {
+    if(isgauss && is.null(varcov) && win$type == "rectangle") {
       # evaluate Gaussian probabilities directly
       xr <- win$xrange
       yr <- win$yrange
@@ -173,7 +286,10 @@ densitypointsEngine <- function(x, sigma, ...,
         pnorm(yr[2], mean=yy, sd=sigma) - pnorm(yr[1], mean=yy, sd=sigma)
       edgeweight <- xprob * yprob
     } else {
-      edg <- second.moment.calc(x, sigma=sigma, what="edge", varcov=varcov)
+      edg <- second.moment.calc(x, sigma=sigma,
+                                kernel=kernel,
+                                scalekernel=scalekernel,
+                                what="edge", varcov=varcov)
       edgeweight <- safelookup(edg, x, warn=FALSE)
     }
     if(diggle) {
@@ -187,9 +303,73 @@ densitypointsEngine <- function(x, sigma, ...,
       }
     }
   }
-  
-  if(spatstat.options("densityC") || k > 1) {
-    # .................. new C code ...........................
+
+  if(isgauss &&
+     spatstat.options("densityTransform") && spatstat.options("densityC")) {
+    ## .................. experimental C code .....................
+    if(debugging)
+      cat('Using experimental code!\n')
+    npts <- npoints(x)
+    result <- if(k == 1) numeric(npts) else matrix(, npts, k)
+    xx <- x$x
+    yy <- x$y
+    ## transform to standard coordinates
+    if(is.null(varcov)) {
+      xx <- xx/(sqrt(2) * sigma)
+      yy <- yy/(sqrt(2) * sigma)
+    } else {
+      xy <- cbind(xx, yy) %*% matrixsqrt(Sinv/2)
+      xx <- xy[,1]
+      yy <- xy[,2]
+      sorted <- FALSE
+    }
+    ## cutoff in standard coordinates
+    cutoff <- cutoff/(sqrt(2) * sd)
+    ## sort into increasing order of x coordinate (required by C code)
+    if(!sorted) {
+      oo <- fave.order(xx)
+      xx <- xx[oo]
+      yy <- yy[oo]
+    }
+    if(is.null(weights)) {
+      zz <- .C("Gdenspt",
+               nxy     = as.integer(npts),
+               x       = as.double(xx),
+               y       = as.double(yy),
+               rmaxi   = as.double(cutoff),
+               result  = as.double(double(npts)))
+      if(sorted) result <- zz$result else result[oo] <- zz$result
+      result <- result * const
+    } else if(k == 1) {
+      wtsort <- if(sorted) weights else weights[oo]
+      zz <- .C("Gwtdenspt",
+               nxy     = as.integer(npts),
+               x       = as.double(xx),
+               y       = as.double(yy),
+               rmaxi   = as.double(cutoff),
+               weight  = as.double(wtsort),
+               result  = as.double(double(npts)))
+      if(sorted) result <- zz$result else result[oo] <- zz$result 
+      result <- result * const
+    } else {
+      ## matrix of weights
+      wtsort <- if(sorted) weights else weights[oo, ]
+      for(j in 1:k) {
+        zz <- .C("Gwtdenspt",
+                 nxy     = as.integer(npts),
+                 x       = as.double(xx),
+                 y       = as.double(yy),
+                 rmaxi   = as.double(cutoff),
+                 weight  = as.double(wtsort[,j]),
+                 result  = as.double(double(npts)))
+        if(sorted) result[,j] <- zz$result else result[oo,j] <- zz$result
+      }
+      result <- result * const
+    }
+  } else if(isgauss && spatstat.options("densityC")) {
+    # .................. C code ...........................
+    if(debugging)
+      cat('Using standard code.\n')
     npts <- npoints(x)
     result <- if(k == 1) numeric(npts) else matrix(, npts, k)
     # sort into increasing order of x coordinate (required by C code)
@@ -201,7 +381,6 @@ densitypointsEngine <- function(x, sigma, ...,
       xx <- x$x[oo]
       yy <- x$y[oo]
     }
-    DUP <- spatstat.options("dupC")
     if(is.null(varcov)) {
       # isotropic kernel
       if(is.null(weights)) {
@@ -211,9 +390,7 @@ densitypointsEngine <- function(x, sigma, ...,
                  y       = as.double(yy),
                  rmaxi   = as.double(cutoff),
                  sig     = as.double(sd),
-                 result  = as.double(double(npts)),
-                 DUP     = DUP)
-#                 PACKAGE = "spatstat")
+                 result  = as.double(double(npts)))
         if(sorted) result <- zz$result else result[oo] <- zz$result 
       } else if(k == 1) {
         wtsort <- if(sorted) weights else weights[oo]
@@ -224,9 +401,7 @@ densitypointsEngine <- function(x, sigma, ...,
                  rmaxi   = as.double(cutoff),
                  sig     = as.double(sd),
                  weight  = as.double(wtsort),
-                 result  = as.double(double(npts)),
-                 DUP     = DUP)
-#                 PACKAGE = "spatstat")
+                 result  = as.double(double(npts)))
         if(sorted) result <- zz$result else result[oo] <- zz$result 
        } else {
         # matrix of weights
@@ -239,9 +414,7 @@ densitypointsEngine <- function(x, sigma, ...,
                    rmaxi   = as.double(cutoff),
                    sig     = as.double(sd),
                    weight  = as.double(wtsort[,j]),
-                   result  = as.double(double(npts)),
-                   DUP     = DUP)
-#                   PACKAGE = "spatstat")
+                   result  = as.double(double(npts)))
           if(sorted) result[,j] <- zz$result else result[oo,j] <- zz$result
         }
       }
@@ -256,9 +429,7 @@ densitypointsEngine <- function(x, sigma, ...,
                  rmaxi   = as.double(cutoff),
                  detsigma = as.double(detSigma),
                  sinv    = as.double(flatSinv),
-                 result  = as.double(double(npts)),
-                 DUP     = DUP)
-#                 PACKAGE = "spatstat")
+                 result  = as.double(double(npts)))
         if(sorted) result <- zz$result else result[oo] <- zz$result 
       } else if(k == 1) {
         # vector of weights
@@ -271,9 +442,7 @@ densitypointsEngine <- function(x, sigma, ...,
                  detsigma = as.double(detSigma),
                  sinv    = as.double(flatSinv),
                  weight  = as.double(wtsort),
-                 result   = as.double(double(npts)),
-                 DUP     = DUP)
-#                 PACKAGE = "spatstat")
+                 result   = as.double(double(npts)))
         if(sorted) result <- zz$result else result[oo] <- zz$result 
       } else {
         # matrix of weights
@@ -287,34 +456,47 @@ densitypointsEngine <- function(x, sigma, ...,
                    detsigma = as.double(detSigma),
                    sinv    = as.double(flatSinv),
                    weight  = as.double(wtsort[,j]),
-                   result  = as.double(double(npts)),
-                   DUP     = DUP)
-#                   PACKAGE = "spatstat")
+                   result  = as.double(double(npts)))
           if(sorted) result[,j] <- zz$result else result[oo,j] <- zz$result 
         }
       }
     }
   } else {
-      # ..... interpreted code .........................................
+    # ..... interpreted code .........................................
     close <- closepairs(x, cutoff)
     i <- close$i
     j <- close$j
     d <- close$d
+    npts <- npoints(x)
+    result <- if(k == 1) numeric(npts) else matrix(, npts, k)
     # evaluate contribution from each close pair (i,j)
-    if(is.null(varcov)) {
-      contrib <- const * exp(-d^2/(2 * sigma^2))
+    if(isgauss) { 
+      if(is.null(varcov)) {
+        contrib <- const * exp(-d^2/(2 * sigma^2))
+      } else {
+        ## anisotropic kernel
+        dx <- close$dx
+        dy <- close$dy
+        contrib <- const * exp(-(dx * (dx * Sinv[1,1] + dy * Sinv[1,2])
+                                 + dy * (dx * Sinv[2,1] + dy * Sinv[2,2]))/2)
+      }
     } else {
-      # anisotropic kernel
-      dx <- close$dx
-      dy <- close$dy
-      contrib <- const * exp(-(dx * (dx * Sinv[1,1] + dy * Sinv[1,2])
-                               + dy * (dx * Sinv[2,1] + dy * Sinv[2,2]))/2)
+      contrib <- evaluate2Dkernel(kernel, close$dx, close$dy,
+                                  sigma=sigma, varcov=varcov, ...)
+    }
+    ## sum (weighted) contributions
+    ifac <- factor(i, levels=1:npts)
+    if(is.null(weights)) {
+      result <- tapply(contrib, ifac, sum)
+    } else if(k == 1) {
+      wcontrib <- contrib * weights[j]
+      result <- tapply(wcontrib, ifac, sum)
+    } else {
+      for(kk in 1:k) {
+        wcontribkk <- contrib * weights[j, kk]
+        result[,kk] <- tapply(wcontribkk, ifac, sum)
+      }
     }
-    # multiply by weights
-    if(!is.null(weights))
-      contrib <- contrib * weights[j]
-    # sum
-    result <- tapply(contrib, factor(i, levels=1:(x$n)), sum)
     result[is.na(result)] <- 0
     #
   }
@@ -339,7 +521,7 @@ densitypointsEngine <- function(x, sigma, ...,
                  "in leave-one-out method:",
                  "length(lambda) = ", length(result),
                  "!=", npts, "= npoints"))
-    if(any(is.na(result))) {
+    if(anyNA(result)) {
       nwrong <- sum(is.na(result))
       stop(paste("Internal error:", nwrong, "NA or NaN",
                  ngettext(nwrong, "value", "values"),
@@ -355,13 +537,16 @@ densitypointsEngine <- function(x, sigma, ...,
                  "in leave-one-out method:",
                  "nrow(lambda) = ", nrow(result),
                  "!=", npts, "= npoints"))
-    if(any(is.na(result))) {
+    if(anyNA(result)) {
       nwrong <- sum(!complete.cases(result))
       stop(paste("Internal error:", nwrong,
                  ngettext(nwrong, "row", "rows"),
                  "of NA values generated in leave-one-out method"))
     }
   }
+  if(spill)
+      return(list(result=result, sigma=sigma, varcov=varcov,
+                  edg=edgeweight))
   # tack on bandwidth
   attr(result, "sigma") <- sigma
   attr(result, "varcov") <- varcov
@@ -432,84 +617,14 @@ resolve.2D.kernel <- function(..., sigma=NULL, varcov=NULL, x, mindist=NULL,
   return(result)
 }
 
-bw.diggle <- function(X, ...) {
-  stopifnot(is.ppp(X))
-  # secret option for debugging
-  mf <- function(..., method=c("C", "interpreted")) match.arg(method)
-  method <- mf(...)
-  #
-  lambda <- npoints(X)/area.owin(X)
-  K <- Kest(X, correction="good")
-  yname <- fvnames(K, ".y")
-  K <- K[, c("r", yname)]
-  rvals <- K$r
-  # evaluation of M(r) requires K(2r)
-  rmax2 <- max(rvals)/2
-  if(!is.null(alim <- attr(K, "alim"))) rmax2 <- min(alim[2], rmax2)
-  ok <- (rvals <= rmax2)
-  switch(method,
-         interpreted = {
-           rvals <- rvals[ok]
-           nr <- length(rvals)
-           J <- numeric(nr)
-           phi <- function(x,h) { 
-             if(h <= 0) return(numeric(length(x)))
-             y <- pmax.int(0, pmin.int(1, x/(2 * h)))
-             4 * pi * h^2 * (acos(y) - y * sqrt(1 - y^2))
-           }
-           for(i in 1:nr) 
-             J[i] <- stieltjes(phi, K, h=rvals[i])[[yname]]/(2 * pi)
-         },
-         C = {
-           nr <- length(rvals)
-           nrmax <- sum(ok)
-           dK <- diff(K[[yname]])
-           ndK <- length(dK)
-           DUP <- spatstat.options("dupC")
-           z <- .C("digberJ",
-                   r=as.double(rvals),
-                   dK=as.double(dK),
-                   nr=as.integer(nr),
-                   nrmax=as.integer(nrmax),
-                   ndK=as.integer(ndK),
-                   J=as.double(numeric(nrmax)),
-                   DUP=DUP)
-#                  PACKAGE="spatstat"              
-           J <- z$J
-           rvals <- rvals[ok]
-         })
-  pir2 <- pi * rvals^2
-  M <- (1/lambda - 2 * K[[yname]][ok])/pir2 + J/pir2^2
-  # This calculation was for the uniform kernel on B(0,h)
-  # Convert to standard deviation of (one-dimensional marginal) kernel
-  sigma <- rvals/2
-  result <- bw.optim(M, sigma,
-                     creator="bw.diggle",
-                     criterion="Berman-Diggle Cross-Validation",
-                     J=J,
-                     lambda=lambda)
-  return(result)
-}
-
-bw.scott <- function(X) {
-  stopifnot(is.ppp(X))
-  n <- npoints(X)
-  sdx <- sqrt(var(X$x))
-  sdy <- sqrt(var(X$y))
-  return(c(sdx, sdy) * n^(-1/6))
-}
-
 
 densitycrossEngine <- function(Xdata, Xquery, sigma, ...,
                                weights=NULL, edge=TRUE, varcov=NULL,
                                diggle=FALSE,
                                sorted=FALSE) {
-  if(is.null(varcov)) {
-    const <- 1/(2 * pi * sigma^2)
-  } else {
+  if(!is.null(varcov)) {
     detSigma <- det(varcov)
     Sinv <- solve(varcov)
-    const <- 1/(2 * pi * sqrt(detSigma))
   }
   if(length(weights) == 0 || (!is.null(dim(weights)) && nrow(weights) == 0))
     weights <- NULL
@@ -589,7 +704,6 @@ densitycrossEngine <- function(Xdata, Xquery, sigma, ...,
     xd <- xd[ood]
     yd <- yd[ood]
   }
-  DUP <- spatstat.options("dupC")
   if(is.null(varcov)) {
     ## isotropic kernel
     if(is.null(weights)) {
@@ -602,8 +716,7 @@ densitycrossEngine <- function(Xdata, Xquery, sigma, ...,
                yd      = as.double(yd),
                rmaxi   = as.double(cutoff),
                sig     = as.double(sd),
-               result  = as.double(double(nquery)),
-               DUP     = DUP)
+               result  = as.double(double(nquery)))
       if(sorted) result <- zz$result else result[ooq] <- zz$result 
     } else if(k == 1) {
       wtsort <- if(sorted) weights else weights[ood]
@@ -617,8 +730,7 @@ densitycrossEngine <- function(Xdata, Xquery, sigma, ...,
                wd      = as.double(wtsort),
                rmaxi   = as.double(cutoff),
                sig     = as.double(sd),
-               result  = as.double(double(nquery)),
-               DUP     = DUP)
+               result  = as.double(double(nquery)))
       if(sorted) result <- zz$result else result[ooq] <- zz$result 
     } else {
       ## matrix of weights
@@ -634,10 +746,10 @@ densitycrossEngine <- function(Xdata, Xquery, sigma, ...,
                  wd      = as.double(wtsort[,j]),
                  rmaxi   = as.double(cutoff),
                  sig     = as.double(sd),
-                 result  = as.double(double(nquery)),
-                 DUP     = DUP)
+                 result  = as.double(double(nquery)))
         if(sorted) result[,j] <- zz$result else result[ooq,j] <- zz$result
       }
+      colnames(result) <- weightnames
     }
   } else {
     ## anisotropic kernel
@@ -653,8 +765,7 @@ densitycrossEngine <- function(Xdata, Xquery, sigma, ...,
                rmaxi   = as.double(cutoff),
                detsigma = as.double(detSigma),
                sinv    = as.double(flatSinv),
-               result  = as.double(double(nquery)),
-               DUP     = DUP)
+               result  = as.double(double(nquery)))
       if(sorted) result <- zz$result else result[ooq] <- zz$result 
     } else if(k == 1) {
       ## vector of weights
@@ -670,8 +781,7 @@ densitycrossEngine <- function(Xdata, Xquery, sigma, ...,
                rmaxi   = as.double(cutoff),
                detsigma = as.double(detSigma),
                sinv    = as.double(flatSinv),
-               result   = as.double(double(nquery)),
-               DUP     = DUP)
+               result   = as.double(double(nquery)))
       if(sorted) result <- zz$result else result[ooq] <- zz$result 
     } else {
       ## matrix of weights
@@ -688,10 +798,10 @@ densitycrossEngine <- function(Xdata, Xquery, sigma, ...,
                  rmaxi   = as.double(cutoff),
                  detsigma = as.double(detSigma),
                  sinv    = as.double(flatSinv),
-                 result  = as.double(double(nquery)),
-                 DUP     = DUP)
+                 result  = as.double(double(nquery)))
         if(sorted) result[,j] <- zz$result else result[ooq,j] <- zz$result 
       }
+      colnames(result) <- weightnames
     }
   }
   # ........  Edge correction ........................................
@@ -704,3 +814,5 @@ densitycrossEngine <- function(Xdata, Xquery, sigma, ...,
   # 
   return(result)
 }
+
+
