@@ -1,7 +1,7 @@
 #
 #       images.R
 #
-#         $Revision: 1.110 $     $Date: 2014/04/21 03:31:59 $
+#      $Revision: 1.140 $     $Date: 2016/12/12 06:06:12 $
 #
 #      The class "im" of raster images
 #
@@ -34,7 +34,7 @@ im <- function(mat, xcol=seq_len(ncol(mat)), yrow=seq_len(nrow(mat)),
   miss.yrow <- missing(yrow)
   
   # determine dimensions
-  if(is.matrix(mat)) {
+  if(!is.null(dim(mat))) {
     nr <- nrow(mat)
     nc <- ncol(mat)
     if(length(xcol) != nc)
@@ -89,11 +89,6 @@ im <- function(mat, xcol=seq_len(ncol(mat)), yrow=seq_len(nrow(mat)),
 
   unitname <- as.units(unitname)
 
-  # get rid of those annoying 8.67e-19 printouts
-  swat <- function(x) {ifelseAX(abs(x) < .Machine$double.eps, 0, x)}
-  xrange <- swat(xrange)
-  yrange <- swat(yrange)
-  
   out <- list(v   = mat,
               dim = c(nr, nc),
               xrange   = xrange,
@@ -151,6 +146,16 @@ shift.im <- function(X, vec=c(0,0), ..., origin=NULL) {
   return(X)
 }
 
+"Frame<-.im" <- function(X, value) {
+  stopifnot(is.rectangle(value))
+  if(!is.subset.owin(value, Frame(X))) {
+    ## first expand
+    X <- X[value, drop=FALSE]
+  }
+  X[value, drop=TRUE]
+}
+
+
 "[.im" <- local({
 
   disjoint <- function(r, s) { (r[2] < s[1]) || (r[1] > s[2])  }
@@ -209,9 +214,24 @@ shift.im <- function(X, vec=c(0,0), ..., origin=NULL) {
         ##                 as an image (if 'i' is a rectangle)
         ##                 or as a vector (otherwise)
 
-        out <- if(is.null(raster)) x else as.im(raster)
-        xy <- expand.grid(y=out$yrow,x=out$xcol)
+        ## determine pixel raster for output
         if(!is.null(raster)) {
+          out <- as.im(raster)
+          do.resample <- TRUE
+        } else if(is.subset.owin(i, as.owin(x))) {
+          out <- x
+          do.resample <- FALSE
+        } else {
+          ## new window does not contain data window: expand it
+          bb <- boundingbox(as.rectangle(i), as.rectangle(x))
+          rr <- if(is.mask(i)) i else x
+          xcol <- prolongseq(rr$xcol, bb$xrange, rr$xstep)
+          yrow <- prolongseq(rr$yrow, bb$yrange, rr$ystep)
+          out <- list(xcol=xcol, yrow=yrow)
+          do.resample <- TRUE
+        }
+        xy <- expand.grid(y=out$yrow,x=out$xcol)
+        if(do.resample) {
           ## resample image on new pixel raster
           values <- lookup.im(x, xy$x, xy$y, naok=TRUE)
           out <- im(values, out$xcol, out$yrow, unitname=unitname(out))
@@ -245,7 +265,7 @@ shift.im <- function(X, vec=c(0,0), ..., origin=NULL) {
           if(ncolsub > 1) list(xcol = out$xcol[colsub]) else list(xrange=xr)
         yarg <-
           if(nrowsub > 1) list(yrow = out$yrow[rowsub]) else list(yrange=yr)
-        result <- do.call("im", c(marg, xarg, yarg))
+        result <- do.call(im, c(marg, xarg, yarg))
         return(result)
       }
       if(verifyclass(i, "im", fatal=FALSE)) {
@@ -259,6 +279,17 @@ shift.im <- function(X, vec=c(0,0), ..., origin=NULL) {
         } else stop("Subset argument \'i\' is an image, but not of logical type")
       }
 
+      if(inherits(i, "linnet")) {
+        #' linear network
+        if(jtype == "given")
+          warning("Argument j ignored")
+        W <- raster %orifnull% as.owin(x)
+        M <- as.mask.psp(as.psp(i), W=W, ...)
+        xM <- x[M, drop=drop]
+        if(is.im(xM)) xM <- linim(i, xM)
+        return(xM)
+      }
+      
       if(is.ppp(i)) {
         ## 'i' is a point pattern 
         if(jtype == "given")
@@ -288,6 +319,9 @@ shift.im <- function(X, vec=c(0,0), ..., origin=NULL) {
 
     ## Construct a matrix index call for possible re-use
     M <- as.matrix(x)
+    ## suppress warnings from code checkers
+    dont.complain.about(M)
+    ##
     ycall <- switch(itype,
                     given = {
                       switch(jtype,
@@ -372,9 +406,25 @@ shift.im <- function(X, vec=c(0,0), ..., origin=NULL) {
   Extract.im
 })
 
+update.im <- function(object, ...) {
+  ## update internal structure of image after manipulation
+  X <- object
+  mat <- X$v
+  typ <- typeof(mat)
+  if(typ == "double")
+    typ <- "real"
+  ## deal with factor case
+  if(is.factor(mat)) {
+    typ <- "factor"
+  } else if(!is.null(lev <- levels(mat))) {
+    typ <- "factor"
+    X$v <- factor(mat, levels=lev)
+  }
+  X$type <- typ
+  return(X)
+}
 
 "[<-.im" <- function(x, i, j, value) {
-  
   # detect 'blank' arguments like second argument of x[i, ] 
   ngiven <- length(sys.call())
   nmatched <- length(match.call())
@@ -390,18 +440,25 @@ shift.im <- function(X, vec=c(0,0), ..., origin=NULL) {
 
   X <- x
   W <- as.owin(X)
-  if(is.im(value)) {
+
+  stopifnot(is.im(value) || is.vector(value) ||
+            is.matrix(value) || is.array(value) || is.factor(value))
+  if(is.im(value)) 
     value <- value$v
-  }
-  stopifnot(is.vector(value) || is.matrix(value) || is.factor(value))
 
   if(itype == "missing" && jtype == "missing") {
     # no index provided
     # set all pixels to 'value'
     v <- X$v
-    v[!is.na(v)] <- value
+    if(!is.factor(value)) {
+      v[!is.na(v)] <- value
+    } else {
+      vnew <- matrix(NA_integer_, ncol(v), nrow(v))
+      vnew[!is.na(v)] <- as.integer(value)
+      v <- factor(vnew, labels=levels(value))
+    }
     X$v <- v
-    return(X)
+    return(update(X))
   }
   if(itype == "given") {
     # ..................... Try a spatial index ....................
@@ -410,22 +467,26 @@ shift.im <- function(X, vec=c(0,0), ..., origin=NULL) {
       # 'i' is a window
       if(is.empty(i))
         return(X)
-      xx <- as.vector(raster.x(W))
-      yy <- as.vector(raster.y(W))
+      rxy <- rasterxy.mask(W)
+      xx <- rxy$x
+      yy <- rxy$y
       ok <- inside.owin(xx, yy, i)
       X$v[ok] <- value
-      return(X)
+      X$type <- ifelse(is.factor(X$v), "factor", typeof(X$v))
+      return(update(X))
     }
     if(verifyclass(i, "im", fatal=FALSE) && i$type == "logical") {
       if(jtype == "given") warning("Index j ignored")
       # convert logical vector to window where entries are TRUE
       i <- as.owin(eval.im(ifelse1NA(i)))
       # continue as above
-      xx <- as.vector(raster.x(W))
-      yy <- as.vector(raster.y(W))
+      rxy <- rasterxy.mask(W)
+      xx <- rxy$x
+      yy <- rxy$y
       ok <- inside.owin(xx, yy, i)
       X$v[ok] <- value
-      return(X)
+      X$type <- ifelse(is.factor(X$v), "factor", typeof(X$v))
+      return(update(X))
     }
     if(is.ppp(i)) {
       # 'i' is a point pattern
@@ -448,7 +509,8 @@ shift.im <- function(X, vec=c(0,0), ..., origin=NULL) {
         # set values
         X$v[cbind(loc$row, loc$col)] <- value
       }
-      return(X)
+      X$type <- ifelse(is.factor(X$v), "factor", typeof(X$v))
+      return(update(X))
     }
   }
   # .................. 'i' is not a spatial index ....................
@@ -475,9 +537,10 @@ shift.im <- function(X, vec=c(0,0), ..., origin=NULL) {
                   })
   # try it
   litmus <- try(eval(as.call(ycall)), silent=TRUE)
-  if(!inherits(litmus, "try-error")) 
-    return(X)
-
+  if(!inherits(litmus, "try-error")){
+    X$type <- ifelse(is.factor(X$v), "factor", typeof(X$v))
+    return(update(X))
+  }
   #  Last chance!
   if(itype == "given" &&
      !is.matrix(i) &&
@@ -502,7 +565,8 @@ shift.im <- function(X, vec=c(0,0), ..., origin=NULL) {
       # set values
       X$v[cbind(loc$row, loc$col)] <- value
     }
-    return(X)
+    X$type <- ifelse(is.factor(X$v), "factor", typeof(X$v))
+    return(update(X))
   }
 
   stop("The subset operation is undefined for this type of index")
@@ -617,42 +681,72 @@ lookup.im <- function(Z, x, y, naok=FALSE, strict=TRUE) {
   # insert into answer
   value[frameok] <- vf
 
-  if(!naok && any(is.na(value)))
+  if(!naok && anyNA(value))
     warning("Internal error: NA's generated")
 
   return(value)
 }
   
 
+## low level
+
 rasterx.im <- function(x) {
   verifyclass(x, "im")
-  v <- x$v
   xx <- x$xcol
-  matrix(xx[col(v)], ncol=ncol(v), nrow=nrow(v))
+  matrix(xx[col(x)], ncol=ncol(x), nrow=nrow(x))
 }
 
 rastery.im <- function(x) {
   verifyclass(x, "im")
-  v <- x$v
   yy <- x$yrow
-  matrix(yy[row(v)], ncol=ncol(v), nrow=nrow(v))
+  matrix(yy[row(x)], ncol=ncol(x), nrow=nrow(x))
 }
 
 rasterxy.im <- function(x, drop=FALSE) {
   verifyclass(x, "im")
-  v <- x$v
   xx <- x$xcol
   yy <- x$yrow
-  if(!drop) {
-    ans <- cbind(as.vector(xx[col(v)]), as.vector(yy[row(v)]))
-  } else {
-    ok <- !is.null(x$v)
-    ans <- cbind(as.vector(xx[col(v)[ok]]), as.vector(yy[row(v)[ok]]))
+  ans <- cbind(x=as.vector(xx[col(x)]),
+               y=as.vector(yy[row(x)]))
+  if(drop) {
+    ok <- as.vector(!is.na(x$v))
+    ans <- ans[ok, , drop=FALSE]
   }
-  colnames(ans) <- c("x", "y")
   return(ans)
 }
 
+## user interface 
+
+raster.x <- function(w, drop=FALSE) {
+  if(is.owin(w)) return(rasterx.mask(w, drop=drop))
+  if(!is.im(w)) stop("w should be a window or an image")
+  x <- w$xcol[col(w)]
+  x <- if(drop) x[!is.na(w$v), drop=TRUE] else array(x, dim=w$dim)
+  return(x)
+}
+  
+raster.y <- function(w, drop=FALSE) {
+  if(is.owin(w)) return(rastery.mask(w, drop=drop))
+  if(!is.im(w)) stop("w should be a window or an image")
+  y <- w$yrow[row(w)]
+  y <- if(drop) y[!is.na(w$v), drop=TRUE] else array(y, dim=w$dim)
+  return(y)
+}
+
+raster.xy <- function(w, drop=FALSE) {
+  if(is.owin(w)) return(rasterxy.mask(w, drop=drop))
+  if(!is.im(w)) stop("w should be a window or an image")
+  y <- w$xcol[col(w)]
+  y <- w$yrow[row(w)]
+  if(drop) {
+    ok <- !is.na(w$v)
+    x <- x[ok, drop=TRUE]
+    y <- y[ok, drop=TRUE]
+  }
+  return(list(x=as.numeric(x),
+              y=as.numeric(y)))
+}
+
 ##############
 
 # methods for other functions
@@ -722,6 +816,54 @@ min.im <- function(x, ...) {
   return(min(xvalues, ...))
 }
 
+where.max <- function(x, first=TRUE) {
+  stopifnot(is.im(x))
+  if(first) { 
+    ## find the first maximum
+    v <- x$v
+    locn <- which.max(as.vector(v))  # ignores NA, NaN
+    locrow <- as.vector(row(v))[locn]
+    loccol <- as.vector(col(v))[locn]
+  } else {
+    ## find all maxima
+    xmax <- max(x)
+    M <- solutionset(x == xmax)
+    loc <- which(M$m, arr.ind=TRUE)
+    locrow <- loc[,1]
+    loccol <- loc[,2]
+  }
+  xx <- x$xcol[loccol]
+  yy <- x$yrow[locrow]
+  return(ppp(x=xx, y=yy, window=Window(x)))
+}
+
+where.min <- function(x, first=TRUE) {
+  stopifnot(is.im(x))
+  if(first) { 
+    ## find the first minimum
+    v <- x$v
+    locn <- which.min(as.vector(v))  # ignores NA, NaN
+    locrow <- as.vector(row(v))[locn]
+    loccol <- as.vector(col(v))[locn]
+  } else {
+    ## find all minima
+    xmin <- min(x)
+    M <- solutionset(x == xmin)
+    loc <- which(M$m, arr.ind=TRUE)
+    locrow <- loc[,1]
+    loccol <- loc[,2]
+  }
+  xx <- x$xcol[loccol]
+  yy <- x$yrow[locrow]
+  return(ppp(x=xx, y=yy, window=Window(x)))
+}
+
+## the following ensures that 'sd' works
+
+as.double.im <- function(x, ...) { as.double(x[], ...) }
+
+##
+
 hist.im <- function(x, ..., probability=FALSE) {
   xname <- short.deparse(substitute(x))
   verifyclass(x, "im")
@@ -742,7 +884,7 @@ hist.im <- function(x, ..., probability=FALSE) {
       heights <- tab
       ylab <- "Number of pixels"
     }
-    mids <- do.call("barplot",
+    mids <- do.call(barplot,
                    resolve.defaults(list(heights),
                                     list(...),
                                     list(xlab=paste("Pixel value"),
@@ -757,17 +899,17 @@ hist.im <- function(x, ..., probability=FALSE) {
     plotit <- resolve.defaults(list(...), list(plot=TRUE))$plot
     if(plotit) {
       ylab <- if(probability) "Probability density" else "Number of pixels"
-      out <- do.call("hist.default",
-                   resolve.defaults(list(values),
-                                    list(...),
-                                    list(probability=probability),
-                                    list(xlab=paste("Pixel value"),
-                                         ylab=ylab,
-                                         main=main)))
+      out <- do.call(hist.default,
+                     resolve.defaults(list(values),
+                                      list(...),
+                                      list(freq=!probability,
+                                           xlab="Pixel value",
+                                           ylab=ylab,
+                                           main=main)))
       out$xname <- xname
     } else {
       # plot.default whinges if `probability' given when plot=FALSE
-      out <- do.call("hist.default",
+      out <- do.call(hist.default,
                    resolve.defaults(list(values),
                                     list(...)))
       # hack!
@@ -778,7 +920,7 @@ hist.im <- function(x, ..., probability=FALSE) {
 }
 
 plot.barplotdata <- function(x, ...) {
-  do.call("barplot",
+  do.call(barplot,
           resolve.defaults(list(height=x$heights),
                            list(...),
                            list(main=paste("Histogram of ", x$xname))))
@@ -799,19 +941,27 @@ cut.im <- function(x, ...) {
 
 quantile.im <- function(x, ...) {
   verifyclass(x, "im")
-  q <- do.call("quantile",
+  q <- do.call(quantile,
                resolve.defaults(list(as.numeric(as.matrix(x))),
                                 list(...),
                                 list(na.rm=TRUE)))
   return(q)
 }
 
-integral.im <- function(x, ...) {
-  verifyclass(x, "im")
-  typ <- x$type
+integral <- function(f, domain=NULL, ...) {
+  UseMethod("integral")
+}
+
+integral.im <- function(f, domain=NULL, ...) {
+  verifyclass(f, "im")
+  typ <- f$type
   if(!any(typ == c("integer", "real", "complex", "logical")))
     stop(paste("Don't know how to integrate an image of type", sQuote(typ)))
-  a <- with(x, sum(v, na.rm=TRUE) * xstep * ystep)
+  if(!is.null(domain)) {
+    if(is.tess(domain)) return(sapply(tiles(domain), integral.im, f=f))
+    f <- f[domain, drop=FALSE, tight=TRUE]
+  }
+  a <- with(f, sum(v, na.rm=TRUE) * xstep * ystep)
   return(a)
 }
 
@@ -845,14 +995,14 @@ split.im <- function(x, f, ..., drop=FALSE) {
   if(drop)
     return(out)
   else 
-    return(as.listof(out))
+    return(as.solist(out))
 }
 
 by.im <- function(data, INDICES, FUN, ...) {
   stopifnot(is.im(data))
   V <- split(data, INDICES)
   U <- lapply(V, FUN, ...)
-  return(as.listof(U))
+  return(as.solist(U, demote=TRUE))
 }
 
 rebound.im <- function(x, rect) {
@@ -906,13 +1056,31 @@ sort.im <- function(x, ...) {
 dim.im <- function(x) { x$dim }
 
 # colour images
-rgbim <- function(R, G, B, maxColorValue=255) {
-  eval.im(factor(rgbNA(as.vector(R), as.vector(G), as.vector(B),
-                     maxColorValue=maxColorValue)))
+rgbim <- function(R, G, B, A=NULL, maxColorValue=255, autoscale=FALSE) {
+  if(autoscale) {
+    R <- scaletointerval(R, 0, maxColorValue)
+    G <- scaletointerval(G, 0, maxColorValue)
+    B <- scaletointerval(B, 0, maxColorValue)
+    if(!is.null(A))
+      A <- scaletointerval(A, 0, maxColorValue)
+  }
+  Z <- eval.im(factor(rgbNA(as.vector(R), as.vector(G), as.vector(B),
+                            as.vector(A),
+                            maxColorValue=maxColorValue)))
+  return(Z)
 }
 
-hsvim <- function(H, S, V) {
-  eval.im(factor(hsvNA(as.vector(H), as.vector(S), as.vector(V))))
+hsvim <- function(H, S, V, A=NULL, autoscale=FALSE) {
+  if(autoscale) {
+    H <- scaletointerval(H, 0, 1)
+    S <- scaletointerval(S, 0, 1)
+    V <- scaletointerval(V, 0, 1)
+    if(!is.null(A))
+      A <- scaletointerval(A, 0, 1)
+  }
+  Z <- eval.im(factor(hsvNA(as.vector(H), as.vector(S), as.vector(V),
+                            as.vector(A))))
+  return(Z)
 }
 
 scaletointerval <- function(x, from=0, to=1, xrange=range(x)) {
@@ -923,7 +1091,12 @@ scaletointerval.default <- function(x, from=0, to=1, xrange=range(x)) {
   x <- as.numeric(x)
   rr <- if(missing(xrange)) range(x, na.rm=TRUE) else as.numeric(xrange)
   b <- as.numeric(to - from)/diff(rr)
-  y <- from + b * (x - rr[1])
+  if(is.finite(b)) {
+    y <- from + b * (x - rr[1])
+  } else {
+    y <- (from+to)/2 + 0 * x
+  }
+  y[] <- pmin(pmax(y[], from), to)
   return(y)
 }
 
@@ -939,3 +1112,83 @@ zapsmall.im <- function(x, digits) {
   return(eval.im(zapsmall(x, digits=digits)))
 }
 
+domain.im <- Window.im <- function(X, ...) { as.owin(X) }
+
+"Window<-.im" <- function(X, ..., value) {
+  verifyclass(value, "owin")
+  X[value, drop=FALSE]
+}
+
+padimage <- function(X, value=NA, n=1, W=NULL) {
+  stopifnot(is.im(X))
+  stopifnot(length(value) == 1)
+  if(!missing(n) && !is.null(W)) 
+    stop("Arguments n and W are incompatible", call.=FALSE)
+  padW <- !is.null(W)
+  if(isfac <- (X$type == "factor")) {
+    ## handle factors
+    levX <- levels(X)
+    if(is.factor(value)) {
+      stopifnot(identical(levels(X), levels(value)))
+    } else {
+      value <- factor(value, levels=levX)
+    }
+    X <- eval.im(as.integer(X))
+    value <- as.integer(value)
+  }
+  if(!padW) {
+    ## pad by 'n' pixels
+    nn <- rep(n, 4)
+    nleft   <- nn[1]
+    nright  <- nn[2]
+    nbottom <- nn[3]
+    ntop    <- nn[4]
+  } else {
+    ## pad out to window W
+    FX <- Frame(X)
+    B <- boundingbox(Frame(W), FX)
+    nleft   <- max(1, round((FX$xrange[1] - B$xrange[1])/X$xstep))
+    nright  <- max(1, round((B$xrange[2] - FX$xrange[2])/X$xstep))
+    nbottom <- max(1, round((FX$yrange[1] - B$yrange[1])/X$ystep))
+    ntop    <- max(1, round((B$yrange[2] - FX$yrange[2])/X$ystep))
+  }
+  mX <- as.matrix(X)
+  dd <- dim(mX)
+  mX <- cbind(matrix(value, dd[1], nleft, byrow=TRUE),
+              as.matrix(X),
+              matrix(value, dd[1], nright, byrow=TRUE))
+  dd <- dim(mX)
+  mX <- rbind(matrix(rev(value), nbottom, dd[2]),
+              mX,
+              matrix(value, ntop, dd[2]))
+  xcol <- with(X,
+               c(xcol[1]     - (nleft:1) * xstep,
+                 xcol,
+                 xcol[length(xcol)] + (1:nright) * xstep))
+  yrow <- with(X,
+               c(yrow[1]     - (nbottom:1) * ystep,
+                 yrow,
+                 yrow[length(yrow)] + (1:ntop) * ystep))
+  xr <- with(X, xrange + c(-nleft, nright) * xstep)
+  yr <- with(X, yrange + c(-nbottom, ntop) * ystep)
+  Y <- im(mX,
+          xcol=xcol, yrow=yrow, xrange=xr, yrange=yr,
+          unitname=unitname(X))
+  if(isfac)
+    Y <- eval.im(factor(Y, levels=seq_along(levX), labels=levX))
+  if(padW && !is.rectangle(W)) 
+    Y <- Y[W, drop=FALSE]
+  return(Y)
+}
+
+as.function.im <- function(x, ...) {
+  Z <- x
+  f <- function(x,y) { Z[list(x=x, y=y)] }
+  g <- funxy(f, Window(x))
+  return(g)
+}
+
+anyNA.im <- function(x, recursive=FALSE) {
+  anyNA(x$v)
+}
+
