@@ -3,7 +3,7 @@
 #
 #	The 'plot' method for observation windows (class "owin")
 #
-#	$Revision: 1.46 $	$Date: 2014/05/05 06:37:17 $
+#	$Revision: 1.58 $	$Date: 2016/07/16 06:11:47 $
 #
 #
 #
@@ -12,7 +12,8 @@ plot.owin <- function(x, main, add=FALSE, ..., box, edge=0.04,
                       type = c("w", "n"), show.all=!add,
                       hatch=FALSE,
                       hatchargs=list(), 
-                      invert=FALSE, do.plot=TRUE)
+                      invert=FALSE, do.plot=TRUE,
+                      claim.title.space=FALSE) 
 {
 #
 # Function plot.owin.  A method for plot.
@@ -32,54 +33,68 @@ plot.owin <- function(x, main, add=FALSE, ..., box, edge=0.04,
   } else stopifnot(is.logical(box) && length(box) == 1)
 
 ####
-  if(is.expression(main)) 
-    nlines <- 1
-  else {
-    # convert to string and count number of lines
-    main <- paste(main)
-    if(length(main) > 1)
-      main <- paste(main, collapse="\n")
-    if(nchar(main) == 0)
-      nlines <- 0
-    else
-      nlines <- length(strsplit(main, "\n")[[1]])
-  }
+  pt <- prepareTitle(main)
+  main <- pt$main
+  nlines <- pt$nlines
 #########        
   xlim <- xr <- W$xrange
   ylim <- yr <- W$yrange
 
-####################################################  
+####################################################
+
+  ## graphics parameters that can be overridden by user
+  gparam <- resolve.defaults(list(...), par())
+  ## character expansion factors
+  ##     main title size = 'cex.main' * par(cex.main) * par(cex)
+  ## user's graphics expansion factor (*multiplies* par)
+  cex.main.user <- resolve.1.default(list(cex.main=1), list(...))
+  ## size of main title as multiple of par('cex')
+  cex.main.rela <- cex.main.user * par('cex.main') 
+  ## absolute size
+  cex.main.absol <- cex.main.rela * par('cex')
+    
   if(!add) {
-    # new plot
-    # allow space for main title
-    guesslinespace <- 0.08 * diff(yr)
-    ylim[2] <- ylim[2] + nlines * guesslinespace
-    # set up plot with equal scales
-    do.call.plotfun("plot.default",
+    ## new plot
+    if(claim.title.space && nlines > 0) {
+      ## allow space for main title (only in multi-panel plots)
+      guesslinespace <- 0.07 * sqrt(diff(xr)^2 + diff(yr)^2) * cex.main.absol
+      added <- (nlines + 1) * guesslinespace
+      ylim[2] <- ylim[2] + added
+    }
+    ## set up plot with equal scales
+    do.call.plotfun(plot.default,
                     resolve.defaults(list(x=numeric(0), y=numeric(0),
                                           type="n"),
                                      list(...),
                                      list(xlim=xlim, ylim=ylim,
                                           ann=FALSE, axes=FALSE,
-                                          asp=1.0)))
+                                          asp=1.0,
+                                          xaxs="i", yaxs="i"),
+                                     .MatchNull=FALSE))
   }
-  if(show.all) {
-    ## add title in a reasonable place!
-    if(nlines > 0) {
-      parval <- resolve.defaults(list(...), par())
-      mainheight <- strheight(main, units="user", cex=parval$cex.main)
-      gapheight <- (strheight("b\nb", units="user", cex=parval$cex.main)
-                    - 2 * strheight("b", units="user", cex=parval$cex.main))
+  if(show.all && nlines > 0) {
+    ## add title 
+    if(claim.title.space) {
+      mainheight <- sum(strheight(main, units="user", cex=cex.main.rela))
+      gapheight <- (strheight("b\nb", units="user", cex=cex.main.rela)
+                    - 2 * strheight("b", units="user", cex=cex.main.rela))
+      if(nlines > 1 && !is.expression(main))
+        main <- paste(main, collapse="\n")
       text(x=mean(xr), y=yr[2] + mainheight + 0.5 * gapheight, labels=main,
-           cex=parval$cex.main,
-           col=parval$col.main,
-           font=parval$font.main)
+           cex=cex.main.rela,
+           col=gparam$col.main,
+           font=gparam$font.main)
+    } else {
+      title(main=main,
+            cex=cex.main.rela,
+            col=gparam$col.main,
+            font=gparam$font.main)
     }
   }
   
 # Draw surrounding box
   if(box)
-    do.call.plotfun("segments",
+    do.call.plotfun(segments,
                     resolve.defaults(
                                      list(x0=xr[c(1,2,2,1)],
                                           y0=yr[c(1,1,2,2)],
@@ -98,12 +113,12 @@ plot.owin <- function(x, main, add=FALSE, ..., box, edge=0.04,
          rectangle = {
            Wpoly <- as.polygonal(W)
            po <- Wpoly$bdry[[1]]
-           do.call.plotfun("polygon",
+           do.call.plotfun(polygon,
                            resolve.defaults(list(x=po),
                                             list(...)),
                            extrargs="lwd")
            if(hatch)
-             do.call("add.texture", append(list(W=W), hatchargs))
+             do.call(add.texture, append(list(W=W), hatchargs))
          },
          polygonal = {
            p <- W$bdry
@@ -128,7 +143,7 @@ plot.owin <- function(x, main, add=FALSE, ..., box, edge=0.04,
              # No triangulation required;
              # simply plot the polygons
              for(i in seq_along(p))
-               do.call.plotfun("polygon",
+               do.call.plotfun(polygon,
                                resolve.defaults(
                                                 list(x=p[[i]]),
                                                 list(...)),
@@ -137,10 +152,9 @@ plot.owin <- function(x, main, add=FALSE, ..., box, edge=0.04,
               # Try using polypath():
              lucy <- names(dev.cur())
              if(!(lucy %in% c("xfig","pictex","X11"))) {
-               xx <- unlist(lapply(p, function(a) {c(NA, a$x)}))[-1]
-               yy <- unlist(lapply(p, function(a) {c(NA, a$y)}))[-1]
-               do.call.plotfun("polypath",
-                               resolve.defaults(list(x=xx,y=yy),
+               ppa <- owin2polypath(W)
+               do.call.plotfun(polypath,
+                               resolve.defaults(ppa,
                                                 list(border=col.poly),
                                                 list(...)))
              } else {
@@ -152,7 +166,7 @@ plot.owin <- function(x, main, add=FALSE, ..., box, edge=0.04,
                  # Fill pieces with colour (and draw border in same colour)
                  pp <- broken$bdry
                  for(i in seq_len(length(pp)))
-                   do.call.plotfun("polygon",
+                   do.call.plotfun(polygon,
                                    resolve.defaults(list(x=pp[[i]],
                                                          border=col.poly),
                                                     list(...)))
@@ -160,7 +174,7 @@ plot.owin <- function(x, main, add=FALSE, ..., box, edge=0.04,
              }
              # Now draw polygon boundaries
              for(i in seq_along(p))
-               do.call.plotfun("polygon",
+               do.call.plotfun(polygon,
                                resolve.defaults(
                                                 list(x=p[[i]]),
                                                 list(density=0, col=NA),
@@ -168,7 +182,7 @@ plot.owin <- function(x, main, add=FALSE, ..., box, edge=0.04,
                                extrargs="lwd")
            }
            if(hatch)
-             do.call("add.texture", append(list(W=W), hatchargs))
+             do.call(add.texture, append(list(W=W), hatchargs))
          },
          mask = {
            # capture 'col' argument and ensure it's at least 2 values
@@ -190,7 +204,7 @@ plot.owin <- function(x, main, add=FALSE, ..., box, edge=0.04,
            if(spatstat.options("monochrome"))
              col <- to.grey(col)
            
-           do.call.matched("image.default",
+           do.call.matched(image.default,
                            resolve.defaults(
                            list(x=W$xcol, y=W$yrow, z=t(W$m), add=TRUE),
                            list(col=col),       
@@ -198,7 +212,7 @@ plot.owin <- function(x, main, add=FALSE, ..., box, edge=0.04,
                            spatstat.options("par.binary"),
                            list(zlim=c(FALSE, TRUE))))
            if(hatch)
-             do.call("add.texture", append(list(W=W), hatchargs))
+             do.call(add.texture, append(list(W=W), hatchargs))
          },
          stop(paste("Don't know how to plot window of type", sQuote(W$type)))
          )
@@ -216,7 +230,7 @@ break.holes <- local({
     if(length(ab)==0)
       return(emptywindow(Box))
     # ensure correct polarity
-    totarea <- sum(unlist(lapply(ab, area.xypolygon)))
+    totarea <- sum(unlist(lapply(ab, Area.xypolygon)))
     if(totarea < 0)
       ab <- lapply(ab, reverse.xypolygon)
     AB <- owin(Box$xrange, Box$yrange,
