@@ -3,7 +3,7 @@
 #
 #  Smooth the marks of a point pattern
 # 
-#  $Revision: 1.20 $  $Date: 2014/04/04 03:04:03 $
+#  $Revision: 1.35 $  $Date: 2016/11/13 01:55:53 $
 #
 
 smooth.ppp <- function(X, ..., weights=rep(1, npoints(X)), at="pixels") {
@@ -16,6 +16,10 @@ Smooth <- function(X, ...) {
   UseMethod("Smooth")
 }
 
+Smooth.solist <- function(X, ...) {
+  solapply(X, Smooth, ...)
+}
+
 Smooth.ppp <- function(X, sigma=NULL, ...,
                        weights=rep(1, npoints(X)), at="pixels",
                        edge=TRUE, diggle=FALSE) {
@@ -51,10 +55,19 @@ Smooth.ppp <- function(X, sigma=NULL, ...,
     return(Y)
   }
   ## weights
-  weightsgiven <- !missing(weights) && !is.null(weights) && (length(weights)>0)
+  weightsgiven <- !missing(weights) && !is.null(weights) 
+  if(weightsgiven) {
+    if(is.im(weights)) {
+      weights <- safelookup(weights, X) # includes warning if NA
+    } else if(is.expression(weights)) 
+      weights <- eval(weights, envir=as.data.frame(X), enclos=parent.frame())
+    if(length(weights) == 0 || (!is.null(dim(weights)) && nrow(weights) == 0))
+      weightsgiven <- FALSE
+  }
   if(weightsgiven) {
     check.nvector(weights, npoints(X))
   } else weights <- NULL
+
   if(diggle) {
     ## absorb Diggle edge correction into weights vector
     edg <- second.moment.calc(X, sigma, what="edge", ..., varcov=varcov)
@@ -76,101 +89,128 @@ Smooth.ppp <- function(X, sigma=NULL, ...,
       warning("Factor valued marks were converted to integers")
       values <- as.numeric(values)
     }
-    switch(at,
-           points={
-             result <-
-               do.call("smoothpointsEngine",
-                       resolve.defaults(list(x=X,
-                                             values=values, weights=weights,
-                                             sigma=sigma, varcov=varcov,
-                                             edge=FALSE),
-                                        list(...)))
-           },
-           pixels={
-             values.weights <- if(weightsgiven) values * weights else values
-             numerator <-
-               do.call("density.ppp",
-                       resolve.defaults(list(x=X,
-                                             at="pixels",
-                                             weights = values.weights,
-                                             sigma=sigma, varcov=varcov,
-                                             edge=FALSE),
-                                        list(...)))
-             denominator <-
-               do.call("density.ppp",
-                       resolve.defaults(list(x=X,
-                                             at="pixels",
-                                             weights = weights,
-                                             sigma=sigma,
-                                             varcov=varcov,
-                                             edge=FALSE),
-                                        list(...)))
-             result <- eval.im(numerator/denominator)
-             # trap small values of denominator
-             # trap NaN and +/- Inf values of result, but not NA
-             eps <- .Machine$double.eps
-             nbg <- eval.im(is.infinite(result)
-                            | is.nan(result)
-                            | (denominator < eps))
-             if(any(as.matrix(nbg), na.rm=TRUE)) {
-               warning("Numerical underflow detected: sigma is probably too small")
-               # l'Hopital's rule
-               distX <- distmap(X, xy=numerator)
-               whichnn <- attr(distX, "index")
-               nnvalues <- eval.im(values[whichnn])
-               result[nbg] <- nnvalues[nbg]
-             }
-             attr(result, "warnings") <- attr(numerator, "warnings")
-           })
-  } else {
-    # ......... data frame of marks ..................
-    nmarx <- ncol(marx)
-    # compute denominator
-    denominator <-
-      do.call("density.ppp",
-              resolve.defaults(list(x=X,
-                                    at=at,
-                                    weights = weights,
-                                    sigma=sigma, varcov=varcov,
-                                    edge=FALSE),
-                               list(...)))
-    # compute numerator for each column of marks
-    marx.weights <- if(weightsgiven) marx * weights else marx
-    numerators <-
-      do.call("density.ppp",
-              resolve.defaults(list(x=X,
-                                    at=at,
-                                    weights = marx.weights,
-                                    sigma=sigma, varcov=varcov,
-                                    edge=FALSE),
-                               list(...)))
-    uhoh <- attr(numerators, "warnings")
-    # calculate ratios
-    switch(at,
-           points={
-             if(is.null(uhoh)) {
-               # numerators is a matrix
-               ratio <- numerators/denominator
-               if(any(badpoints <- apply(!is.finite(ratio), 1, any))) {
-                 whichnnX <- nnwhich(X)
-                 ratio[badpoints,] <- marx[whichnnX[badpoints], ]
-               }
-             } else {
-               warning("returning original values")
-               ratio <- marx
-             }
-             result <- as.data.frame(ratio)
-             colnames(result) <- colnames(marx)
-           },
+    ## detect constant values
+    ra <- range(values, na.rm=TRUE)
+    if(diff(ra) == 0) {
+      switch(at,
+             points = {
+               result <- values
+             },
+             pixels = {
+               M <- do.call.matched(as.mask, list(w=as.owin(X), ...))
+               result <- as.im(ra[1], M)
+             })
+    } else {
+      switch(at,
+             points={
+               result <-
+                 do.call(smoothpointsEngine,
+                         resolve.defaults(list(x=X,
+                                               values=values, weights=weights,
+                                               sigma=sigma, varcov=varcov,
+                                               edge=FALSE),
+                                          list(...)))
+             },
              pixels={
-               ratio <- lapply(numerators,
-                               function(a,b) eval.im(a/b),
-                               b=denominator)
+               values.weights <- if(weightsgiven) values * weights else values
+               numerator <-
+                 do.call(density.ppp,
+                         resolve.defaults(list(x=X,
+                                               at="pixels",
+                                               weights = values.weights,
+                                               sigma=sigma, varcov=varcov,
+                                               edge=FALSE),
+                                          list(...)))
+               denominator <-
+                 do.call(density.ppp,
+                         resolve.defaults(list(x=X,
+                                               at="pixels",
+                                               weights = weights,
+                                               sigma=sigma,
+                                               varcov=varcov,
+                                               edge=FALSE),
+                                          list(...)))
+               result <- eval.im(numerator/denominator)
+               ## trap small values of denominator
+               ## trap NaN and +/- Inf values of result, but not NA
+               eps <- .Machine$double.eps
+               nbg <- eval.im(is.infinite(result)
+                              | is.nan(result)
+                              | (denominator < eps))
+               if(any(as.matrix(nbg), na.rm=TRUE)) {
+                 warning(paste("Numerical underflow detected:",
+                               "sigma is probably too small"))
+                 ## l'Hopital's rule
+                 distX <- distmap(X, xy=numerator)
+                 whichnn <- attr(distX, "index")
+                 nnvalues <- eval.im(values[whichnn])
+                 result[nbg] <- nnvalues[nbg]
+               }
+               attr(result, "warnings") <- attr(numerator, "warnings")
+             })
+    }
+  } else {
+    ## ......... data frame of marks ..................
+    ## detect constant columns
+    ra <- apply(marx, 2, range, na.rm=TRUE)
+    isconst <- (apply(ra, 2, diff) == 0)
+    if(anyisconst <- any(isconst)) {
+      oldmarx <- marx
+#      oldX <- X
+      marx <- marx[, !isconst]
+      X <- X %mark% marx
+    }
+    if(any(!isconst)) {
+      ## compute denominator
+      denominator <-
+        do.call(density.ppp,
+                resolve.defaults(list(x=X,
+                                      at=at,
+                                      weights = weights,
+                                      sigma=sigma, varcov=varcov,
+                                      edge=FALSE),
+                                 list(...)))
+      ## compute numerator for each column of marks
+      marx.weights <- if(weightsgiven) marx * weights else marx
+      numerators <-
+        do.call(density.ppp,
+                resolve.defaults(list(x=X,
+                                      at=at,
+                                      weights = marx.weights,
+                                      sigma=sigma, varcov=varcov,
+                                      edge=FALSE),
+                                 list(...)))
+      uhoh <- attr(numerators, "warnings")
+      ## calculate ratios
+      switch(at,
+             points={
+               if(is.null(uhoh)) {
+                 ## numerators is a matrix (or may have dropped to vector)
+                 if(!is.matrix(numerators))
+                   numerators <- matrix(numerators, ncol=1)
+                 ratio <- numerators/denominator
+                 if(any(badpoints <- apply(!is.finite(ratio), 1, any))) {
+                   whichnnX <- nnwhich(X)
+                   ratio[badpoints,] <-
+                     as.matrix(marx[whichnnX[badpoints], , drop=FALSE])
+                 }
+               } else {
+                 warning("returning original values")
+                 ratio <- marx
+               }
+               result <- as.data.frame(ratio)
+               colnames(result) <- colnames(marx)
+             },
+             pixels={
+               ## numerators is a list of images (or may have dropped to 'im')
+               if(is.im(numerators))
+                 numerators <- list(numerators)
+               ratio <- lapply(numerators, "/", e2=denominator)
                if(!is.null(uhoh)) {
-                 # compute nearest neighbour map on same raster
+                 ## compute nearest neighbour map on same raster
                  distX <- distmap(X, xy=denominator)
                  whichnnX <- attr(distX, "index")
-                 # fix images
+                 ## fix images
                  for(j in 1:length(ratio)) {
                    ratj <- ratio[[j]]
                    valj <- marx[,j]
@@ -179,13 +219,38 @@ Smooth.ppp <- function(X, sigma=NULL, ...,
                  }
                  attr(ratio, "warnings") <- uhoh
                }
-               result <- as.listof(ratio)
+               result <- as.solist(ratio)
                names(result) <- colnames(marx)
-           })
+             })
+    } else result <- NULL 
+    if(anyisconst) {
+      partresult <- result
+      switch(at,
+             points = {
+               nX <- npoints(X)
+               result <- matrix(, nX, ncol(oldmarx))
+               if(length(partresult) > 0)
+                 result[,!isconst] <- as.matrix(partresult)
+               result[,isconst] <- rep(ra[1,isconst], each=nX)
+               colnames(result) <- colnames(oldmarx)
+             },
+             pixels = {
+               result <- vector(mode="list", length=ncol(oldmarx))
+               if(length(partresult) > 0) {
+                 result[!isconst] <- partresult
+                 M <- as.owin(partresult[[1]])
+               } else {
+                 M <- do.call.matched(as.mask, list(w=as.owin(X), ...))
+               }
+               result[isconst] <- lapply(ra[1, isconst], as.im, W=M)
+               result <- as.solist(result)
+               names(result) <- colnames(oldmarx)
+             })
     }
-    # wrap up
+  }
+  ## wrap up
   attr(result, "warnings") <-
-    unlist(lapply(result, function(x){ attr(x, "warnings") }))
+    unlist(lapply(result, attr, which="warnings"))
   attr(result, "sigma") <- sigma
   attr(result, "varcov") <- varcov
   return(result)
@@ -195,20 +260,24 @@ Smooth.ppp <- function(X, sigma=NULL, ...,
 smoothpointsEngine <- function(x, values, sigma, ...,
                                weights=NULL, varcov=NULL,
                                leaveoneout=TRUE,
-                               sorted=FALSE) {
+                               sorted=FALSE, cutoff=NULL) {
+  debugging <- spatstat.options("developer")
   stopifnot(is.logical(leaveoneout))
-  if(is.null(varcov)) {
-    const <- 1/(2 * pi * sigma^2)
-  } else {
-    detSigma <- det(varcov)
-    Sinv <- solve(varcov)
-    const <- 1/(2 * pi * sqrt(detSigma))
+  #' detect constant values
+  if(diff(range(values, na.rm=TRUE)) == 0) { 
+    result <- values
+    attr(result, "sigma") <- sigma
+    attr(result, "varcov") <- varcov
+    return(result)
   }
-  # Contributions from pairs of distinct points
-  # closer than 8 standard deviations
+  #' Contributions from pairs of distinct points
+  #' closer than 8 standard deviations
   sd <- if(is.null(varcov)) sigma else sqrt(sum(diag(varcov)))
-  cutoff <- 8 * sd
-
+  if(is.null(cutoff)) 
+    cutoff <- 8 * sd
+  if(debugging)
+    cat(paste("cutoff=", cutoff, "\n"))
+  
   ## Handle weights that are meant to be null
   if(length(weights) == 0 || (!is.null(dim(weights)) && nrow(weights) == 0))
      weights <- NULL
@@ -233,8 +302,68 @@ smoothpointsEngine <- function(x, values, sigma, ...,
     # ensure cutoff includes at least one point
     cutoff <- max(1.1 * nnrange[2], cutoff)
   }
-  if(spatstat.options("densityC")) {
-    # .................. new C code ...........................
+  if(spatstat.options("densityTransform") && spatstat.options("densityC")) {
+    ## .................. experimental C code .....................
+    if(debugging)
+      cat('Using experimental code!\n')
+    npts <- npoints(x)
+    result <- numeric(npts)
+    ## transform to standard coordinates
+    xx <- x$x
+    yy <- x$y
+    if(is.null(varcov)) {
+      xx <- xx/(sqrt(2) * sigma)
+      yy <- yy/(sqrt(2) * sigma)
+    } else {
+      Sinv <- solve(varcov)
+      xy <- cbind(xx, yy) %*% matrixsqrt(Sinv/2)
+      xx <- xy[,1]
+      yy <- xy[,2]
+      sorted <- FALSE
+    }
+    ## cutoff in standard coordinates
+    cutoff <- cutoff/(sqrt(2) * sd)
+    ## sort into increasing order of x coordinate (required by C code)
+    if(!sorted) {
+      oo <- fave.order(xx)
+      xx <- xx[oo]
+      yy <- yy[oo]
+      vv <- values[oo]
+    } else {
+      vv <- values
+    }
+    if(is.null(weights)) {
+      zz <- .C("Gsmoopt",
+               nxy     = as.integer(npts),
+               x       = as.double(xx),
+               y       = as.double(yy),
+               v       = as.double(vv),
+               self    = as.integer(!leaveoneout),
+               rmaxi   = as.double(cutoff),
+               result  = as.double(double(npts)))
+      if(sorted) result <- zz$result else result[oo] <- zz$result
+    } else {
+      wtsort <- weights[oo]
+      zz <- .C("Gwtsmoopt",
+               nxy     = as.integer(npts),
+               x       = as.double(xx),
+               y       = as.double(yy),
+               v       = as.double(vv),
+               self    = as.integer(!leaveoneout),
+               rmaxi   = as.double(cutoff),
+               weight  = as.double(wtsort),
+               result  = as.double(double(npts)))
+      if(sorted) result <- zz$result else result[oo] <- zz$result
+    }
+    if(any(nbg <- (is.infinite(result) | is.nan(result)))) {
+      # NaN or +/-Inf can occur if bandwidth is small
+      # Use mark of nearest neighbour (by l'Hopital's rule)
+      result[nbg] <- values[nnwhich(x)[nbg]]
+    }
+  } else if(spatstat.options("densityC")) {
+    # .................. C code ...........................
+    if(debugging)
+      cat('Using standard code.\n')
     npts <- npoints(x)
     result <- numeric(npts)
     # sort into increasing order of x coordinate (required by C code)
@@ -248,7 +377,6 @@ smoothpointsEngine <- function(x, values, sigma, ...,
       yy <- x$y[oo]
       vv <- values[oo]
     }
-    DUP <- spatstat.options("dupC")
     if(is.null(varcov)) {
       # isotropic kernel
       if(is.null(weights)) {
@@ -260,8 +388,7 @@ smoothpointsEngine <- function(x, values, sigma, ...,
                  self    = as.integer(!leaveoneout),
                  rmaxi   = as.double(cutoff),
                  sig     = as.double(sd),
-                 result  = as.double(double(npts)),
-                 DUP     = DUP)
+                 result  = as.double(double(npts)))
         if(sorted) result <- zz$result else result[oo] <- zz$result
       } else {
         wtsort <- weights[oo]
@@ -274,12 +401,12 @@ smoothpointsEngine <- function(x, values, sigma, ...,
                  rmaxi   = as.double(cutoff),
                  sig     = as.double(sd),
                  weight  = as.double(wtsort),
-                 result  = as.double(double(npts)),
-                 DUP     = DUP)
+                 result  = as.double(double(npts)))
         if(sorted) result <- zz$result else result[oo] <- zz$result
       }
     } else {
       # anisotropic kernel
+      Sinv <- solve(varcov)
       flatSinv <- as.vector(t(Sinv))
       if(is.null(weights)) {
         zz <- .C("asmoopt",
@@ -290,8 +417,7 @@ smoothpointsEngine <- function(x, values, sigma, ...,
                  self    = as.integer(!leaveoneout),
                  rmaxi   = as.double(cutoff),
                  sinv    = as.double(flatSinv),
-                 result  = as.double(double(npts)),
-                 DUP     = DUP)
+                 result  = as.double(double(npts)))
         if(sorted) result <- zz$result else result[oo] <- zz$result
       } else {
         wtsort <- weights[oo]
@@ -304,8 +430,7 @@ smoothpointsEngine <- function(x, values, sigma, ...,
                  rmaxi   = as.double(cutoff),
                  sinv    = as.double(flatSinv),
                  weight  = as.double(wtsort),
-                 result  = as.double(double(npts)),
-                 DUP     = DUP)
+                 result  = as.double(double(npts)))
         if(sorted) result <- zz$result else result[oo] <- zz$result
       }
     }
@@ -319,7 +444,7 @@ smoothpointsEngine <- function(x, values, sigma, ...,
     # compute weighted densities
     if(is.null(weights)) {
       # weights are implicitly equal to 1
-      numerator <- do.call("density.ppp",
+      numerator <- do.call(density.ppp,
                          resolve.defaults(list(x=x, at="points"),
                                           list(weights = values),
                                           list(sigma=sigma, varcov=varcov),
@@ -327,7 +452,7 @@ smoothpointsEngine <- function(x, values, sigma, ...,
                                           list(sorted=sorted),
                                           list(...),
                                           list(edge=FALSE)))
-      denominator <- do.call("density.ppp",
+      denominator <- do.call(density.ppp,
                              resolve.defaults(list(x=x, at="points"),
                                               list(sigma=sigma, varcov=varcov),
                                               list(leaveoneout=leaveoneout),
@@ -335,7 +460,7 @@ smoothpointsEngine <- function(x, values, sigma, ...,
                                               list(...),
                                               list(edge=FALSE)))
     } else {
-      numerator <- do.call("density.ppp",
+      numerator <- do.call(density.ppp,
                            resolve.defaults(list(x=x, at="points"),
                                             list(weights = values * weights),
                                             list(sigma=sigma, varcov=varcov),
@@ -343,7 +468,7 @@ smoothpointsEngine <- function(x, values, sigma, ...,
                                             list(sorted=sorted),
                                             list(...),
                                             list(edge=FALSE)))
-      denominator <- do.call("density.ppp",
+      denominator <- do.call(density.ppp,
                              resolve.defaults(list(x=x, at="points"),
                                               list(weights = weights),
                                               list(sigma=sigma, varcov=varcov),
@@ -397,10 +522,10 @@ bw.smoothppp <- function(X, nh=spatstat.options("n.bandwidth"),
   #
   marx <- marks(X)
   # determine a range of bandwidth values
-  n <- npoints(X)
+#  n <- npoints(X)
   if(is.null(hmin) || is.null(hmax)) {
-    W <- as.owin(X)
-    a <- area.owin(W)
+    W <- Window(X)
+#    a <- area(W)
     d <- diameter(as.rectangle(W))
     # Stoyan's rule of thumb 
     stoyan <- bw.stoyan(X)
@@ -417,7 +542,7 @@ bw.smoothppp <- function(X, nh=spatstat.options("n.bandwidth"),
     }
   } else stopifnot(hmin < hmax)
   #
-  h <- exp(seq(from=log(hmin), to=log(hmax), length.out=nh))
+  h <- geomseq(from=hmin, to=hmax, length.out=nh)
   cv <- numeric(nh)
   # 
   # compute cross-validation criterion
@@ -429,7 +554,7 @@ bw.smoothppp <- function(X, nh=spatstat.options("n.bandwidth"),
 
   # optimize
   iopt <- which.min(cv)
-  hopt <- h[iopt]
+#  hopt <- h[iopt]
   #
   if(warn && (iopt == nh || iopt == 1)) 
     warning(paste("Cross-validation criterion was minimised at",
@@ -449,13 +574,13 @@ bw.smoothppp <- function(X, nh=spatstat.options("n.bandwidth"),
 smoothcrossEngine <- function(Xdata, Xquery, values, sigma, ...,
                               weights=NULL, varcov=NULL,
                               sorted=FALSE) {
-  if(is.null(varcov)) {
-    const <- 1/(2 * pi * sigma^2)
-  } else {
-    detSigma <- det(varcov)
-    Sinv <- solve(varcov)
-    const <- 1/(2 * pi * sqrt(detSigma))
-  }
+#  if(is.null(varcov)) {
+#    const <- 1/(2 * pi * sigma^2)
+#  } else {
+#    detSigma <- det(varcov)
+#    Sinv <- solve(varcov)
+#    const <- 1/(2 * pi * sqrt(detSigma))
+#  }
   if(!is.null(dim(weights)))
     stop("weights must be a vector")
 
@@ -543,7 +668,6 @@ smoothcrossEngine <- function(Xdata, Xquery, values, sigma, ...,
     yd <- yd[ood]
     vd <- vd[ood] 
   }
-  DUP <- spatstat.options("dupC")
   if(is.null(varcov)) {
     ## isotropic kernel
     if(is.null(weights)) {
@@ -557,8 +681,7 @@ smoothcrossEngine <- function(Xdata, Xquery, values, sigma, ...,
                vd      = as.double(vd),
                rmaxi   = as.double(cutoff),
                sig     = as.double(sd),
-               result  = as.double(double(nquery)),
-               DUP     = DUP)
+               result  = as.double(double(nquery)))
       if(sorted) result <- zz$result else result[ooq] <- zz$result
     } else {
       wtsort <- weights[ood]
@@ -573,12 +696,12 @@ smoothcrossEngine <- function(Xdata, Xquery, values, sigma, ...,
                wd      = as.double(wtsort),
                rmaxi   = as.double(cutoff),
                sig     = as.double(sd),
-               result  = as.double(double(nquery)),
-               DUP     = DUP)
+               result  = as.double(double(nquery)))
         if(sorted) result <- zz$result else result[ooq] <- zz$result
       }
     } else {
       # anisotropic kernel
+      Sinv <- solve(varcov)
       flatSinv <- as.vector(t(Sinv))
       if(is.null(weights)) {
         zz <- .C("acrsmoopt",
@@ -591,8 +714,7 @@ smoothcrossEngine <- function(Xdata, Xquery, values, sigma, ...,
                  vd      = as.double(vd),
                  rmaxi   = as.double(cutoff),
                  sinv    = as.double(flatSinv),
-                 result  = as.double(double(nquery)),
-                 DUP     = DUP)
+                 result  = as.double(double(nquery)))
         if(sorted) result <- zz$result else result[ooq] <- zz$result
       } else {
         wtsort <- weights[ood]
@@ -607,8 +729,7 @@ smoothcrossEngine <- function(Xdata, Xquery, values, sigma, ...,
                  wd      = as.double(wtsort),
                  rmaxi   = as.double(cutoff),
                  sinv    = as.double(flatSinv),
-                 result  = as.double(double(nquery)),
-                 DUP     = DUP)
+                 result  = as.double(double(nquery)))
         if(sorted) result <- zz$result else result[ooq] <- zz$result
       }
     }
