@@ -2,7 +2,7 @@
 #	wingeom.S	Various geometrical computations in windows
 #
 #
-#	$Revision: 4.85 $	$Date: 2014/01/27 07:54:55 $
+#	$Revision: 4.119 $	$Date: 2016/09/11 04:22:53 $
 #
 #
 #
@@ -11,8 +11,12 @@
 
 volume.owin <- function(x) { area.owin(x) }
 
+area <- function(w) UseMethod("area")
+
+area.default <- function(w) area.owin(as.owin(w))
+
 area.owin <- function(w) {
-	w <- as.owin(w)
+    stopifnot(is.owin(w))
         switch(w$type,
                rectangle = {
 		width <- abs(diff(w$xrange))
@@ -20,7 +24,7 @@ area.owin <- function(w) {
 		area <- width * height
                },
                polygonal = {
-                 area <- sum(unlist(lapply(w$bdry, area.xypolygon)))
+                 area <- sum(unlist(lapply(w$bdry, Area.xypolygon)))
                },
                mask = {
                  pixelarea <- abs(w$xstep * w$ystep)
@@ -51,6 +55,11 @@ perimeter <- function(w) {
   return(NA)
 }
 
+framebottomleft <- function(w) {
+  f <- Frame(w)
+  c(f$xrange[1], f$yrange[1])
+}
+
 sidelengths.owin <- function(x) {
   if(x$type != "rectangle")
     warning("Computing the side lengths of a non-rectangular window")
@@ -59,8 +68,10 @@ sidelengths.owin <- function(x) {
 
 shortside.owin <- function(x) { min(sidelengths(x)) }
 
-eroded.areas <- function(w, r) {
+eroded.areas <- function(w, r, subset=NULL) {
   w <- as.owin(w)
+  if(!is.null(subset) && !is.mask(w))
+    w <- as.mask(w)
   switch(w$type,
          rectangle = {
            width <- abs(diff(w$xrange))
@@ -68,23 +79,24 @@ eroded.areas <- function(w, r) {
            areas <- pmax(width - 2 * r, 0) * pmax(height - 2 * r, 0)
          },
          polygonal = {
-           # warning("Approximating polygonal window by digital image")
+           ## warning("Approximating polygonal window by digital image")
            w <- as.mask(w)
            areas <- eroded.areas(w, r)
          },
          mask = {
-           # distances from each pixel to window boundary
-           b <- bdist.pixels(w, style="matrix")
-           # histogram breaks to satisfy hist()
+           ## distances from each pixel to window boundary
+           b <- if(is.null(subset)) bdist.pixels(w, style="matrix") else 
+                bdist.pixels(w)[subset, drop=TRUE, rescue=FALSE]
+           ## histogram breaks to satisfy hist()
            Bmax <- max(b, r)
            breaks <- c(-1,r,Bmax+1)
-           # histogram of boundary distances
+           ## histogram of boundary distances
            h <- hist(b, breaks=breaks, plot=FALSE)$counts
-           # reverse cumulative histogram
+           ## reverse cumulative histogram
            H <- revcumsum(h)
-           # drop first entry corresponding to r=-1
+           ## drop first entry corresponding to r=-1
            H <- H[-1]
-           # convert count to area
+           ## convert count to area
            pixarea <- w$xstep * w$ystep
            areas <- pixarea * H
          },
@@ -101,9 +113,9 @@ even.breaks.owin <- function(w) {
 
 unit.square <- function() { owin(c(0,1),c(0,1)) }
 
-square <- function(r=1) {
+square <- function(r=1, unitname=NULL) {
   stopifnot(is.numeric(r))
-  if(any(is.na(r) | !is.finite(r)))
+  if(!all(is.finite(r)))
     stop("argument r is NA or infinite")
   if(length(r) == 1) {
     stopifnot(r > 0)
@@ -111,7 +123,7 @@ square <- function(r=1) {
   } else if(length(r) == 2) {
     stopifnot(r[1] < r[2])
   } else stop("argument r must be a single number, or a vector of length 2")
-  owin(r,r)
+  owin(r,r, unitname=unitname)
 }
 
 overlap.owin <- function(A, B) {
@@ -142,21 +154,24 @@ overlap.owin <- function(A, B) {
     for(i in seq_along(AA))
       for(j in seq_along(BB))
         area <- area + overlap.xypolygon(AA[[i]], BB[[j]])
-    return(area)
+    # small negative numbers can occur due to numerical error
+    return(max(0, area))
   }
   if(At=="mask") {
     # count pixels in A that belong to B
     pixelarea <- abs(A$xstep * A$ystep)
-    x <- as.vector(raster.x(A)[A$m])
-    y <- as.vector(raster.y(A)[A$m])
+    rxy <- rasterxy.mask(A, drop=TRUE)
+    x <- rxy$x
+    y <- rxy$y
     ok <- inside.owin(x, y, B) 
     return(pixelarea * sum(ok))
   }
   if(Bt== "mask") {
     # count pixels in B that belong to A
     pixelarea <- abs(B$xstep * B$ystep)
-    x <- as.vector(raster.x(B)[B$m])
-    y <- as.vector(raster.y(B)[B$m])
+    rxy <- rasterxy.mask(B, drop=TRUE)
+    x <- rxy$x
+    y <- rxy$y
     ok <- inside.owin(x, y, A)
     return(pixelarea * sum(ok))
   }
@@ -185,28 +200,56 @@ function(x, i, ...) {
 #
 #
 
-intersect.owin <- function(A, B, ..., fatal=TRUE) {
-  liszt <- list(...)
+intersect.owin <- function(..., fatal=TRUE, p) {
+  argh <- list(...)
+  ## p is a list of arguments to polyclip::polyclip
+  if(missing(p) || is.null(p)) p <- list()
+  ## handle 'solist' objects
+  argh <- expandSpecialLists(argh, "solist")
   rasterinfo <- list()
-  if(length(liszt) > 0) {
+  if(length(argh) > 0) {
     # explicit arguments controlling raster info
-    israster <- names(liszt) %in% names(formals(as.mask))
-    rasterinfo <- liszt[israster]
-    # handle intersection of more than two windows
-    isowin <- unlist(lapply(liszt, is.owin))
-    nextra <- sum(isowin)
-    if(nextra > 0) {
-      windows <- liszt[isowin]
-      for(i in 1:nextra)
-        B <- do.call("intersect.owin",
-                     append(list(B, windows[[i]]), rasterinfo))
+    israster <- names(argh) %in% names(formals(as.mask))
+    if(any(israster)) {
+      rasterinfo <- argh[israster]
+      ## remaining arguments
+      argh <- argh[!israster]
     }
   }
-  if(missing(A) || is.null(A)) return(B)
-  if(missing(B) || is.null(B)) return(A)
-  verifyclass(A, "owin")
-  verifyclass(B, "owin")
-  #
+  ## look for window arguments
+  isowin <- sapply(argh, is.owin)
+  if(any(!isowin))
+    warning("Some arguments were not windows")
+  argh <- argh[isowin]
+  nwin <- length(argh)
+
+  if(nwin == 0) {
+    warning("No windows were given")
+    return(NULL)
+  }
+
+  ## at least one window
+  A <- argh[[1]]
+  if(nwin == 1) return(A)
+  ## at least two windows
+  B <- argh[[2]]
+  
+  if(nwin > 2) {
+    ## handle union of more than two windows
+    windows <- argh[-c(1,2)]
+    ## determine a common set of parameters for polyclip
+    p <- commonPolyclipArgs(A, B, do.call(boundingbox, windows), p=p)
+    ## absorb all windows into B
+    for(i in seq_along(windows))
+      B <- do.call(intersect.owin,
+                   append(list(B, windows[[i]], p=p),
+                            rasterinfo))
+  }
+
+  ## There are now only two windows
+  if(is.empty(A)) return(A)
+  if(is.empty(B)) return(B)
+
   if(identical(A, B))
     return(A)
 
@@ -228,8 +271,8 @@ intersect.owin <- function(A, B, ..., fatal=TRUE) {
   
   Arect <- is.rectangle(A)
   Brect <- is.rectangle(B)
-  Apoly <- is.polygonal(A)
-  Bpoly <- is.polygonal(B)
+#  Apoly <- is.polygonal(A)
+#  Bpoly <- is.polygonal(B)
   Amask <- is.mask(A)
   Bmask <- is.mask(B)
   
@@ -241,12 +284,14 @@ intersect.owin <- function(A, B, ..., fatal=TRUE) {
     ####### Result is polygonal ############
     a <- lapply(as.polygonal(A)$bdry, reverse.xypolygon)
     b <- lapply(as.polygonal(B)$bdry, reverse.xypolygon)
-    ab <- polyclip::polyclip(a, b, "intersection",
-                             fillA="nonzero", fillB="nonzero")
+    ab <- do.call(polyclip::polyclip,
+                  append(list(a, b, "intersection",
+                              fillA="nonzero", fillB="nonzero"),
+                         p))
     if(length(ab)==0)
       return(emptywindow(C))
     # ensure correct polarity
-    totarea <- sum(unlist(lapply(ab, area.xypolygon)))
+    totarea <- sum(unlist(lapply(ab, Area.xypolygon)))
     if(totarea < 0)
       ab <- lapply(ab, reverse.xypolygon)
     AB <- owin(poly=ab, check=FALSE, unitname=unitname(A))
@@ -270,10 +315,10 @@ intersect.owin <- function(A, B, ..., fatal=TRUE) {
   if(length(rasterinfo) > 0) {
     # convert to masks with specified parameters, and intersect
     if(Amask) {
-      A <- do.call("as.mask", append(list(A), rasterinfo))
+      A <- do.call(as.mask, append(list(A), rasterinfo))
       return(restrict.mask(A, B))
     } else {
-      B <- do.call("as.mask", append(list(B), rasterinfo))
+      B <- do.call(as.mask, append(list(B), rasterinfo))
       return(restrict.mask(B, A))
     }
   } 
@@ -311,61 +356,107 @@ intersect.owin <- function(A, B, ..., fatal=TRUE) {
 }
 
 
-union.owin <- function(A, B, ...) {
-  liszt <- list(...)
+union.owin <- function(..., p) {
+  argh <- list(...)
+  ## weed out NULL arguments
+  argh <- argh[!sapply(argh, is.null)]
+  ## p is a list of arguments to polyclip::polyclip
+  if(missing(p) || is.null(p)) p <- list()
+  ## handle 'solist' objects
+  argh <- expandSpecialLists(argh, "solist")
   rasterinfo <- list()
-  if(length(liszt) > 0) {
-    # explicit arguments controlling raster info
-    israster <- names(liszt) %in% names(formals(as.mask))
-    rasterinfo <- liszt[israster]
-    # handle intersection of more than two windows
-    isowin <- unlist(lapply(liszt, is.owin))
-    nextra <- sum(isowin)
-    if(nextra > 0) {
-      windows <- liszt[isowin]
-      for(i in 1:nextra)
-        B <- do.call("union.owin",
-                     append(list(B, windows[[i]]), rasterinfo))
+  if(length(argh) > 0) {
+    ## arguments controlling raster info
+    israster <- names(argh) %in% names(formals(as.mask))
+    if(any(israster)) {
+      rasterinfo <- argh[israster]
+      ## remaining arguments
+      argh <- argh[!israster]
     }
   }
-  #
-  if(missing(A) || is.null(A) || is.empty(A)) return(B)
-  if(missing(B) || is.null(B) || is.empty(B)) return(A)
-  verifyclass(A, "owin")
-  verifyclass(B, "owin")
+  ## look for window arguments
+  isowin <- sapply(argh, is.owin)
+  if(any(!isowin))
+    warning("Some arguments were not windows")
+  argh <- argh[isowin]
+  ## 
+  nwin <- length(argh)
+  if(nwin == 0) {
+    warning("No windows were given")
+    return(NULL)
+  }
+  ## find non-empty ones
+  if(any(isemp <- sapply(argh, is.empty)))
+    argh <- argh[!isemp]
+  nwin <- length(argh)
+  if(nwin == 0) {
+    warning("All windows were empty")
+    return(NULL)
+  }
+  ## at least one window
+  A <- argh[[1]]
+  if(nwin == 1) return(A)
 
+  ## more than two windows
+  if(nwin > 2) {
+    ## check if we need polyclip
+    somepoly <- !all(sapply(argh, is.mask))
+    if(somepoly) {
+      ## determine a common set of parameters for polyclip
+      p <- commonPolyclipArgs(do.call(boundingbox, argh), p=p)
+      ## apply these parameters now to avoid numerical errors
+      argh <- applyPolyclipArgs(argh, p=p)
+      A <- argh[[1]]
+    } 
+    ## absorb all windows into A without rescaling
+    nullp <- list(eps=1, x0=0, y0=0)
+    for(i in 2:nwin) 
+      A <- do.call(union.owin,
+                   append(list(A, argh[[i]], p=nullp),
+                          rasterinfo))
+    if(somepoly) {
+      ## undo rescaling
+      A <- reversePolyclipArgs(A, p=p)
+    }
+    return(A)
+  }
+
+  ## Exactly two windows
+  B <- argh[[2]]
   if(identical(A, B))
     return(A)
 
-  # check units
+  ## check units
   if(!compatible(unitname(A), unitname(B)))
     warning("The two windows have incompatible units of length")
   
-  # Determine type of intersection
+  ## Determine type of intersection
   
-  Arect <- is.rectangle(A)
-  Brect <- is.rectangle(B)
-  Apoly <- is.polygonal(A)
-  Bpoly <- is.polygonal(B)
+##  Arect <- is.rectangle(A)
+##  Brect <- is.rectangle(B)
+##  Apoly <- is.polygonal(A)
+##  Bpoly <- is.polygonal(B)
   Amask <- is.mask(A)
   Bmask <- is.mask(B)
 
-  # Result is not rectangular.
-  # Create a rectangle to contain it.
+  ## Create a rectangle to contain the result
   
   C <- owin(range(A$xrange, B$xrange),
             range(A$yrange, B$yrange),
             unitname=unitname(A))
 
   if(!Amask && !Bmask) {
-    ####### Result is polygonal ############
+    ####### Result is polygonal (or rectangular) ############
     a <- lapply(as.polygonal(A)$bdry, reverse.xypolygon)
     b <- lapply(as.polygonal(B)$bdry, reverse.xypolygon)
-    ab <- polyclip::polyclip(a, b, "union", fillA="nonzero", fillB="nonzero")
+    ab <- do.call(polyclip::polyclip,
+                  append(list(a, b, "union",
+                              fillA="nonzero", fillB="nonzero"),
+                         p))
     if(length(ab) == 0)
       return(emptywindow(C))
-    # ensure correct polarity
-    totarea <- sum(unlist(lapply(ab, area.xypolygon)))
+    ## ensure correct polarity
+    totarea <- sum(unlist(lapply(ab, Area.xypolygon)))
     if(totarea < 0)
       ab <- lapply(ab, reverse.xypolygon)
     AB <- owin(poly=ab, check=FALSE, unitname=unitname(A))
@@ -375,7 +466,7 @@ union.owin <- function(A, B, ...) {
 
   ####### Result is a mask ############
 
-  # Determine pixel raster parameters
+  ## Determine pixel raster parameters
   if(length(rasterinfo) == 0) {
     rasterinfo <-
       if(Amask)
@@ -388,63 +479,72 @@ union.owin <- function(A, B, ...) {
         list()
   }
 
-  # Convert C to mask
-  C <- do.call("as.mask", append(list(w=C), rasterinfo))
-      
-  x <- as.vector(raster.x(C))
-  y <- as.vector(raster.y(C))
+  ## Convert C to mask
+  C <- do.call(as.mask, append(list(w=C), rasterinfo))
+
+  rxy <- rasterxy.mask(C)
+  x <- rxy$x
+  y <- rxy$y
   ok <- inside.owin(x, y, A) | inside.owin(x, y, B)
 
   if(all(ok)) {
-    # result is a rectangle
+    ## result is a rectangle
     C <- as.rectangle(C)
   } else {
-    # result is a mask
+    ## result is a mask
     C$m[] <- ok
   }
   return(C)
 }
 
-setminus.owin <- function(A, B, ...) {
-  if(is.null(A) || is.empty(A)) return(B)
-  if(is.null(B) || is.empty(B)) return(A)
-  verifyclass(A, "owin")
+setminus.owin <- function(A, B, ..., p) {
+  if(is.null(B)) return(A)
   verifyclass(B, "owin")
-
+  if(is.null(A)) return(emptywindow(Frame(B)))
+  verifyclass(A, "owin")
+  if(is.empty(A) || is.empty(B)) return(A)
   if(identical(A, B))
-    return(emptywindow(as.rectangle(A)))
+    return(emptywindow(Frame(A)))
 
-  # check units
+  ## p is a list of arguments to polyclip::polyclip
+  if(missing(p) || is.null(p)) p <- list()
+
+  ## check units
   if(!compatible(unitname(A), unitname(B)))
     warning("The two windows have incompatible units of length")
   
-  # Determine type of arguments
+  ## Determine type of arguments
   
   Arect <- is.rectangle(A)
   Brect <- is.rectangle(B)
-  Apoly <- is.polygonal(A)
-  Bpoly <- is.polygonal(B)
+##  Apoly <- is.polygonal(A)
+##  Bpoly <- is.polygonal(B)
   Amask <- is.mask(A)
   Bmask <- is.mask(B)
 
-  # Case where A and B are both rectangular
+  ## Case where A and B are both rectangular
   if(Arect && Brect) {
+    if(is.subset.owin(A, B))
+      return(emptywindow(B))
     C <- intersect.owin(A, B, fatal=FALSE)
-    if(is.null(C)) return(A)
+    if(is.null(C) || is.empty(C)) return(A)
     return(complement.owin(C, A))
   }
     
-  # Polygonal case
+  ## Polygonal case
 
   if(!Amask && !Bmask) {
     ####### Result is polygonal ############
     a <- lapply(as.polygonal(A)$bdry, reverse.xypolygon)
     b <- lapply(as.polygonal(B)$bdry, reverse.xypolygon)
-    ab <- polyclip::polyclip(a, b, "minus", fillA="nonzero", fillB="nonzero")
+    ab <- do.call(polyclip::polyclip,
+                  append(list(a, b, "minus",
+                              fillA="nonzero", fillB="nonzero"),
+                         p))
     if(length(ab) == 0)
-      return(emptywindow(C))
-    # ensure correct polarity
-    totarea <- sum(unlist(lapply(ab, area.xypolygon)))
+      return(emptywindow(B))
+    ## ensure correct polarity
+    totarea <- sum(unlist(lapply(ab, Area.xypolygon)))
     if(totarea < 0)
       ab <- lapply(ab, reverse.xypolygon)
     AB <- owin(poly=ab, check=FALSE, unitname=unitname(A))
@@ -454,7 +554,7 @@ setminus.owin <- function(A, B, ...) {
 
   ####### Result is a mask ############
 
-  # Determine pixel raster parameters
+  ## Determine pixel raster parameters
   rasterinfo <- 
     if((length(list(...)) > 0))
       list(...)
@@ -467,11 +567,12 @@ setminus.owin <- function(A, B, ...) {
     else
       list()
 
-  # Convert A to mask
-  AB <- do.call("as.mask", append(list(w=A), rasterinfo))
-      
-  x <- as.vector(raster.x(AB))
-  y <- as.vector(raster.y(AB))
+  ## Convert A to mask
+  AB <- do.call(as.mask, append(list(w=A), rasterinfo))
+
+  rxy <- rasterxy.mask(AB)
+  x <- rxy$x
+  y <- rxy$y
   ok <- inside.owin(x, y, A) & !inside.owin(x, y, B)
 
   if(!all(ok))
@@ -482,26 +583,53 @@ setminus.owin <- function(A, B, ...) {
   return(AB)
 }
 
-# auxiliary functions
-  
-trim.mask <- function(M, R, tolerant=TRUE) {
-    # M is a mask,
-    # R is a rectangle
+## auxiliary functions
 
-    # Ensure R is a subset of bounding rectangle of M
+commonPolyclipArgs <- function(..., p=NULL) {
+  # compute a common resolution for polyclip operations
+  # on several windows
+  if(!is.null(p) && !is.null(p$eps) && !is.null(p$x0) && !is.null(p$y0))
+    return(p)
+  bb <- boundingbox(...)
+  xr <- bb$xrange
+  yr <- bb$yrange
+  eps <- p$eps %orifnull% max(diff(xr), diff(yr))/(2^31)
+  x0  <- p$x0  %orifnull% mean(xr)
+  y0  <- p$y0  %orifnull% mean(yr)
+  return(list(eps=eps, x0=x0, y0=y0))
+}
+
+applyPolyclipArgs <- function(x, p=NULL) {
+  if(is.null(p)) return(x)
+  y <- lapply(x, shift, vec=-c(p$x0, p$y0))
+  z <- lapply(y, scalardilate, f=1/p$eps)
+  return(z)
+}
+
+reversePolyclipArgs <- function(x, p=NULL) {
+  if(is.null(p)) return(x)
+  y <- scalardilate(x, f=p$eps)
+  z <- shift(y, vec=c(p$x0, p$y0))
+  return(z)
+}
+
+trim.mask <- function(M, R, tolerant=TRUE) {
+    ## M is a mask,
+    ## R is a rectangle
+
+    ## Ensure R is a subset of bounding rectangle of M
     R <- owin(intersect.ranges(M$xrange, R$xrange),
               intersect.ranges(M$yrange, R$yrange))
     
-    # Deal with very thin rectangles
+    ## Deal with very thin rectangles
     if(tolerant) {
       R$xrange <- adjustthinrange(R$xrange, M$xstep, M$xrange)
       R$yrange <- adjustthinrange(R$yrange, M$ystep, M$yrange)
     }
 
-    # Extract subset of image grid
-    within.range <- function(u, v) { (u >= v[1]) & (u <= v[2]) }
-    yrowok <- within.range(M$yrow, R$yrange)
-    xcolok <- within.range(M$xcol, R$xrange)
+    ## Extract subset of image grid
+    yrowok <- inside.range(M$yrow, R$yrange)
+    xcolok <- inside.range(M$xcol, R$xrange)
     if((ny <- sum(yrowok)) == 0 || (nx <- sum(xcolok)) == 0) 
       return(emptywindow(R))
     Z <- M
@@ -517,17 +645,18 @@ trim.mask <- function(M, R, tolerant=TRUE) {
 }
 
 restrict.mask <- function(M, W) {
-  # M is a mask, W is any window
+  ## M is a mask, W is any window
   stopifnot(is.mask(M))
   stopifnot(inherits(W, "owin"))
-  if(is.rectangle(W) == "rectangle")
+  if(is.rectangle(W))
     return(trim.mask(M, W))
   M <- trim.mask(M, as.rectangle(W))
-  # Determine which pixels of M are inside W
-  Mm <- M$m
-  x <- as.vector(raster.x(M)[Mm])
-  y <- as.vector(raster.y(M)[Mm])
+  ## Determine which pixels of M are inside W
+  rxy <- rasterxy.mask(M, drop=TRUE)
+  x <- rxy$x
+  y <- rxy$y
   ok <- inside.owin(x, y, W)
+  Mm <- M$m
   Mm[Mm] <- ok
   M$m <- Mm
   return(M)
@@ -568,7 +697,13 @@ trim.rectangle <- function(W, xmargin=0, ymargin=xmargin) {
        unitname=unitname(W))
 }
 
-grow.rectangle <- function(W, xmargin=0, ymargin=xmargin) {
+grow.rectangle <- function(W, xmargin=0, ymargin=xmargin, fraction=NULL) {
+  if(!is.null(fraction)) {
+    fraction <- ensure2vector(fraction)
+    if(any(fraction < 0)) stop("fraction must be non-negative")
+    if(missing(xmargin)) xmargin <- fraction[1] * diff(W$xrange)
+    if(missing(ymargin)) ymargin <- fraction[2] * diff(W$yrange)
+  }
   xmargin <- ensure2vector(xmargin)
   ymargin <- ensure2vector(ymargin)
   if(any(xmargin < 0) || any(ymargin < 0))
@@ -591,8 +726,8 @@ grow.mask <- function(M, xmargin=0, ymargin=xmargin) {
   nright <- attr(xcolplus, "nright")
   nbot <- attr(yrowplus, "nleft")
   ntop <- attr(yrowplus, "nright")
-  mplus[ (nleft+1):(length(yrowplus)-nright),
-         (nbot+1):(length(xcolplus)-ntop) ] <- m
+  mplus[ (nbot+1):(length(yrowplus)-ntop),
+         (nleft+1):(length(xcolplus)-nright) ] <- m
   ## pack up
   result <- owin(xrange=Rplus$xrange,
                  yrange=Rplus$yrange,
@@ -609,15 +744,51 @@ bdry.mask <- function(W) {
   m <- W$m
   nr <- nrow(m)
   nc <- ncol(m)
-  b <-     (m != rbind(FALSE,       m[-nr, ]))
-  b <- b | (m != rbind(m[-1, ], FALSE))
-  b <- b | (m != cbind(FALSE,       m[, -nc]))
-  b <- b | (m != cbind(m[, -1], FALSE))
+  if(!spatstat.options('Cbdrymask')) {
+    ## old interpreted code
+    b <-     (m != rbind(FALSE,       m[-nr, ]))
+    b <- b | (m != rbind(m[-1, ], FALSE))
+    b <- b | (m != cbind(FALSE,       m[, -nc]))
+    b <- b | (m != cbind(m[, -1], FALSE))
+  } else {
+    b <- integer(nr * nc)
+    z <- .C("bdrymask",
+            nx = as.integer(nc),
+            ny = as.integer(nr),
+            m = as.integer(m),
+            b = as.integer(b))
+    b <- matrix(as.logical(z$b), nr, nc)
+  }
   W$m <- b
   return(W)
 }
 
+nvertices <- function(x, ...) {
+  UseMethod("nvertices")
+}
+
+nvertices.default <- function(x, ...) {
+  v <- vertices(x)
+  vx <- v$x
+  n <- if(is.null(vx)) NA else length(vx)
+  return(n)
+}
+
+nvertices.owin <- function(x, ...) {
+  if(is.empty(x))
+    return(0)
+  n <- switch(x$type,
+              rectangle=4,
+              polygonal=sum(lengths(lapply(x$bdry, getElement, name="x"))),
+              mask=sum(bdry.mask(x)$m))
+  return(n)
+}
+
 vertices <- function(w) {
+  UseMethod("vertices")
+}
+
+vertices.owin <- function(w) {
   verifyclass(w, "owin")
   if(is.empty(w))
     return(NULL)
@@ -628,14 +799,15 @@ vertices <- function(w) {
            vert <- list(x=xr[c(1,2,2,1)], y=yr[c(1,1,2,2)])
          },
          polygonal={
-           vert <- do.call("concatxy",w$bdry)
+           vert <- do.call(concatxy,w$bdry)
          },
          mask={
-           b <- bdry.mask(w)
-           xx <- raster.x(w)
-           yy <- raster.y(w)
-           vert <- list(x=as.vector(xx[b$m]),
-                        y=as.vector(yy[b$m]))
+           bm <- bdry.mask(w)$m
+           rxy <- rasterxy.mask(w)
+           xx <- rxy$x
+           yy <- rxy$y
+           vert <- list(x=as.vector(xx[bm]),
+                        y=as.vector(yy[bm]))
          })
   return(vert)
 }
@@ -656,6 +828,14 @@ diameter.owin <- function(x) {
   return(sqrt(max(d)))
 }
 
+##    radius of inscribed circle
+
+inradius <- function(W) {
+  stopifnot(is.owin(W))
+  if(W$type == "rectangle") diameter(W)/2 else max(distmap(W, invert=TRUE))
+}
+
+  
 incircle <- function(W) {
   # computes the largest circle contained in W
   verifyclass(W, "owin")
@@ -757,7 +937,7 @@ convexhull <- function(x) {
   if(inherits(x, "owin")) 
     v <- vertices(x)
   else if(inherits(x, "psp"))
-    v <- endpoints.psp
+    v <- endpoints.psp(x)
   else if(inherits(x, "ppp"))
     v <- x
   else {
@@ -791,3 +971,74 @@ emptywindow <- function(w) {
   return(out)
 }
 
+discs <- function(centres, radii=marks(centres)/2, ..., 
+                  separate=FALSE, mask=FALSE, trim=TRUE,
+                  delta=NULL, npoly=NULL) {
+  stopifnot(is.ppp(centres))
+  n <- npoints(centres)
+  if(n == 0) return(emptywindow(Frame(centres)))
+  check.nvector(radii, npoints(centres), oneok=TRUE)
+  stopifnot(all(radii > 0))
+  
+  if(sameradius <- (length(radii) == 1)) 
+    radii <- rep(radii, npoints(centres))
+
+  if(!separate && mask) {
+    #' compute pixel approximation
+    M <- as.mask(Window(centres), ...)
+    z <- .C("discs2grid",
+            nx    = as.integer(M$dim[2]),
+            x0    = as.double(M$xcol[1]),
+            xstep = as.double(M$xstep),  
+            ny    = as.integer(M$dim[1]),
+            y0    = as.double(M$yrow[1]),
+            ystep = as.double(M$ystep), 
+            nd    = as.integer(n),
+            xd    = as.double(centres$x),
+            yd    = as.double(centres$y),
+            rd    = as.double(radii), 
+            out   = as.integer(integer(prod(M$dim))))
+    M$m[] <- as.logical(z$out)
+    return(M)
+  }
+  #' construct a list of discs
+  D <- list()
+  if(!sameradius && length(unique(radii)) > 1) {
+    if(is.null(delta) && is.null(npoly)) {
+      ra <- range(radii)
+      rr <- ra[2]/ra[1]
+      mm <- ceiling(128/rr)
+      mm <- max(16, mm) ## equals 16 unless ra[2]/ra[1] < 8 
+      delta <- 2 * pi * ra[1]/mm
+    }
+    for(i in 1:n)
+      D[[i]] <- disc(centre=centres[i], radius=radii[i],
+                     delta=delta, npoly=npoly)
+  } else {
+    #' congruent discs -- use 'shift'
+    W0 <- disc(centre=c(0,0), radius=radii[1],
+               delta=delta, npoly=npoly)
+    for(i in 1:n) 
+      D[[i]] <- shift(W0, vec=centres[i])
+  } 
+  D <- as.solist(D)
+  #' return list of discs?
+  if(separate)
+    return(D)
+  #' return union of discs
+  W <- union.owin(D)
+  if(trim) W <- intersect.owin(W, Window(centres))
+  return(W)
+}
+
+harmonise.owin <- harmonize.owin <- function(...) {
+  argz <- list(...)
+  wins <- solapply(argz, as.owin)
+  if(length(wins) < 2L) return(wins)
+  ismask <- sapply(wins, is.mask)
+  if(!any(ismask)) return(wins)
+  comgrid <- do.call(commonGrid, lapply(argz, as.owin))
+  result <- solapply(argz, "[", i=comgrid, drop=FALSE)
+  return(result)
+}
+
